import typing
import time
import abc
import urllib.parse
import requests
import os
import datetime
import dateutil.parser
import lxml
import lxml.etree
import posixpath
import pandas
import collections
import presalytics
import presalytics.client.api
import presalytics.story.components
import presalytics.client
import presalytics.lib.exceptions
import presalytics.story.outline
import presalytics.lib.util
import presalytics.lib.plugins.ooxml
import presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_dto
import presalytics.client.presalytics_ooxml_automation.models.table_table_data_dto
if typing.TYPE_CHECKING:
    from presalytics.story.outline import StoryOutline, Page, Widget
    from presalytics.client.presalytics_story import Story as ApiStory
    from presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_dto import ChartChartDataDTO
    from presalytics.client.presalytics_ooxml_automation.models.table_table_data_dto import TableTableDataDTO


class OoxmlEndpointMap(object):
    """
    Mapping class that bridges Presalytics API Ooxml Automation service endpoints
    and component class that consume those endpoints (typically subclasses of 
    `presalytics.lib.widgets.ooxml.OoxmlWidgetBase`)

    The classmethods on this class are conveninece methods to help users 
    quickly inform their widget which endpoint their of a Ooxml Document their
    targets.

    Instance methods on this class are used by widget to generate urls and 
    lookup against object tree for target objects.

    Parameters
    ----------
    endpoint_id : str
        A unique string for identifying the object_type related to the enpoint
    
    baseurl : str
        For developer use. Allows this to generate urls for non-standard instances
        of the Ooxml Automation service.  Defaults to https://api.presalytics.io/ooxml-automation/

    Attributes
    ----------
    root_url : str
        the home url for the class instance.  Typically `http://api.presalytics.io/ooxml-automation/{object_type}`
    
    OBJECT_TYPE_MAP : str
        A mapping table for object_types and object tree lookup keys
    """
    _BASE_URL = "https://api.presalytics.io/ooxml-automation"
    _CHART = "Charts"
    _CONNECTION_SHAPE = "ConnectionShapes"
    _DOCUMENT = "Documents"
    _GROUP = "Groups"
    _IMAGE = "Images"
    _SHAPE = "Shapes"
    _SHAPETREE = "ShapeTrees"
    _SLIDE = "Slides"
    _TABLE = "Tables"
    _THEME = "Themes"

    def __init__(self, endpoint_id, baseurl: str = None):
        if endpoint_id not in OoxmlEndpointMap.__dict__.values():
            raise presalytics.lib.exceptions.ValidationError("{0} is not a valid endpoint ID".format(endpoint_id))
        self.endpoint_id = endpoint_id
        if not baseurl:
            self.baseurl = OoxmlEndpointMap._BASE_URL
            custom_hosts = presalytics.CONFIG.get("HOSTS", None)
            if custom_hosts:
                ooxml_host = custom_hosts.get("OOXML_AUTOMATION", None)
                if ooxml_host:
                    self.baseurl = ooxml_host
                
        else:
            self.baseurl = baseurl
        self.root_url = posixpath.join(self.baseurl, self.endpoint_id)
        self.OBJECT_TYPE_MAP = self._build_object_type_map()
    


    def _build_object_type_map(self):
        return {
            "Chart": [
                OoxmlEndpointMap._CHART,
            ],
            "Slide": [
                OoxmlEndpointMap._GROUP,
                OoxmlEndpointMap._SHAPE,
                OoxmlEndpointMap._SHAPETREE, 
                OoxmlEndpointMap._CONNECTION_SHAPE,
                OoxmlEndpointMap._SLIDE
            ],
            "Table": [
                OoxmlEndpointMap._TABLE
            ],
            "Theme": [
                OoxmlEndpointMap._THEME
            ],
            "Shared": [
                OoxmlEndpointMap._IMAGE
            ],
            "EMPTY": [
                OoxmlEndpointMap._DOCUMENT
            ]
        }
    
    def get_object_type(self):
        """
        Returns the Ooxml Automation service object type for this endpoint
        """
        for key, val in self.OBJECT_TYPE_MAP.items():
            for test_ep in val:
                if test_ep == self.endpoint_id:
                    if key == "EMPTY":
                        return self.endpoint_id
                    else:
                        return "{0}.{1}".format(key, self.endpoint_id)
        message = "Invalid EndpointMap configuration: {0} is not in OBJECT_TYPE_MAP".format(self.endpoint_id)
        raise presalytics.lib.exceptions.ValidationError(message)

    def get_id_url(self, id):
        """
        Returns a url to pull metadata from this Ooxml Automation service endpoint
        """
        return posixpath.join(self.root_url, id)

    def get_svg_url(self, id):
        """
        Returns a url to download an svg from this Ooxml Automation service endpoint
        """
        return posixpath.join(self.root_url, "Svg", id)

    def get_xml_url(self, id):
        """
        Returns a url to pull the Open Office Xml from this Ooxml Automation service endpoint
        """
        return posixpath.join(self.root_url, "OpenOfficeXml", id)

    @classmethod
    def connection_shape(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting ConnectionShape objects
        """
        return cls(OoxmlEndpointMap._CONNECTION_SHAPE, baseurl)

    @classmethod
    def chart(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Chart objects
        """
        return cls(OoxmlEndpointMap._CHART, baseurl)

    @classmethod
    def document(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Document objects
        """
        return cls(OoxmlEndpointMap._DOCUMENT, baseurl)
    
    @classmethod
    def group(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Group objects
        """
        return cls(OoxmlEndpointMap._GROUP, baseurl)

    @classmethod
    def image(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Image objects
        """
        return cls(OoxmlEndpointMap._IMAGE, baseurl)

    @classmethod
    def shape(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Shape objects
        """
        return cls(OoxmlEndpointMap._SHAPE, baseurl)

    @classmethod
    def shapetree(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting ShapeTree objects
        """
        return cls(OoxmlEndpointMap._SHAPETREE, baseurl)

    @classmethod
    def slide(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Slide objects
        """
        return cls(OoxmlEndpointMap._SLIDE, baseurl)

    @classmethod
    def table(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Table objects
        """
        return cls(OoxmlEndpointMap._TABLE, baseurl)

    @classmethod
    def theme(cls, baseurl=None):
        """
        Factory method to create an `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` instance
        targeting Theme objects
        """
        return cls(OoxmlEndpointMap._THEME, baseurl)


class OoxmlWidgetBase(presalytics.story.components.WidgetBase):
    """
    Base class for creating widgets from objects at endpoints in the 
    Presalytics API Ooxml Automation service.

    Parameters
    ----------
    name : str, optional
        The widget name.  If not provided, will be the `object_name` or `filename`

    story_id : str, optional
        The the id of the story in the Presalytics API Story service.  If not provided, 
        a new story will be created.  Do not supply if this object has not yet been created. 
    
    object_ooxml_id : str, optional
        The identifier of the Ooxml Automation service object bound the Story. Do not supply if this 
        object has not yet been created.

    endpoint_map : presalytics.lib.widgets.ooxml.OoxmlEndpointMap, optional
        Reference to the Presalytics API Ooxml Automation service endpoint and object type
        that for the object of interest

    Attributes
    ----------
    svg_html : str
        An html fragment containing the svg data
    """
    endpoint_map: OoxmlEndpointMap
    data: typing.Dict

    __component_kind__ = "ooxml-base-widget"

    __plugins__ = [
        {
            'name': 'external_scripts',
            'kind': 'script',
            'config': {
                'approved_scripts_key': 'ooxml'
            }
        },
        {
            'name': 'external_links',
            'kind': 'style',
            'config': {
                'approved_styles_key': 'preloaders'
            }
        }
    ]

    def __init__(self, 
                 name,
                 story_id=None,
                 object_ooxml_id=None,
                 endpoint_map=None,
                 **kwargs):
        super(OoxmlWidgetBase, self).__init__(name, **kwargs)
        if object_ooxml_id:
            self.object_ooxml_id = object_ooxml_id
        if story_id:
            self.story_id = story_id
        if endpoint_map:
            self.endpoint_map = endpoint_map
        self.svg_html = None

    def create_container(self, **kwargs):
        """
        Wraps the Presalytics API Ooxml Automation service SVG endpoint in an `<iframe>` that
        will be rendered inside of a story
        """
        svg_container_div = lxml.html.Element("div", {
            'class': 'svg-container',
            'data-object-type': self.endpoint_map.endpoint_id,
            'data-object-id': self.object_ooxml_id
        })
        preloader_container_div = lxml.html.Element( "div", {"class":"preloader-container"})
        preloader_row_div = lxml.etree.SubElement(preloader_container_div, "div", attrib={"class":"preloader-row"})
        preloader_file = os.path.join(os.path.dirname(__file__), "img", "preloader.svg")
        svg = lxml.html.parse(preloader_file)
        preloader_row_div.append(svg.getroot())
        empty_parent_div = lxml.html.Element("div", {
            'class': 'empty-parent bg-light'
        })
        empty_parent_div.extend([svg_container_div, preloader_container_div])
        return lxml.html.tostring(empty_parent_div)


    def to_html(self, **kwargs):
        """
        Returns an html string that will render the object at the endpoint
        """
        html = self.create_container()
        try:
            html = html.decode('utf-8')
        except Exception:
            pass
        self.svg_html = html
        return self.svg_html

    def get_svg(self, id, timeout_iterator=0) -> str:
        """
        Get an svg-formatted version of the object from the Ooxml Automation service
        """
        svg_url = self.endpoint_map.get_svg_url(id)
        client = self.get_client()
        headers = client.get_auth_header()
        headers.update(client.get_request_id_header())
        response = requests.get(svg_url, headers=headers)
        svg_data = response.text
        if response.status_code != 200:
            raise presalytics.lib.exceptions.ApiError(message=response.text, status_code=response.status_code)
        if response.text.startswith("Temp data"):
            if timeout_iterator > 5:
                raise presalytics.lib.exceptions.ApiError(message="Unable to download svg.  Please check for upstream processing errors.")
            else:
                time.sleep(2)
                svg_data = self.get_svg(id, timeout_iterator)
        return svg_data

    def get_svg_file(self, filename=None):
        """
        Writes an svg representation of the object to current working directory.  Filname is optional.
        """
        if not filename:
            filename = self.endpoint_map.get_object_type() + "-" + self.object_ooxml_id + ".pptx"
        with open(filename, 'w') as f:
            f.write(self.get_svg(self.object_ooxml_id))

    def serialize(self):
        data = {
            "story_id": self.story_id,
            "object_id": self.object_ooxml_id,
            "endpoint_id": self.endpoint_map.endpoint_id
        }
        widget = presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
            plugins=None
        )
        return widget

    @classmethod
    def deserialize(cls, component, **kwargs):
        return cls(
            component.name, 
            component.data["story_id"], 
            component.data["object_id"],
            OoxmlEndpointMap(component.data["endpoint_id"]),
            **kwargs
        )


class OoxmlFileWidget(OoxmlWidgetBase):
    """
    Builds a `widget` from a Presentation or Spreadsheet document

    This class interacts with the Presalytics API to extract SVG objects from
    Presentation and spreadsheet documents, identify them, and render them 
    into a story. The file is uploaded to Presalytics API Ooxml Automation service,
    which then processes the file and scans for objects in the file's object tree 
    (As seen in the 'Selection Pane' in PowerPoint) for objects matching the 'object_name'.
    When rendered, this widget retrieves an SVG of the identified object for rendering within 
    the story. 

    Please note that the Presalytics API Ooxml Automation object will be created overwritten 
    each time this widget is initialized, and replaced within the corresponding 
    `presalytics.story.outline.StoryOutline`.  For in-place editing of widgets Ooxml Automation objects 
    that are already bound to the `Story`, please see `presalytics.lib.widgets.ooxml_editors.OoxmlEditorWidget`

    Parameters
    ----------
    filename : str
        The local filepath a presentation or spreadsheet file containing
        the object to be rendered

    name : str, optional
        The widget name.  If not provided, attribute will be set as the `object_name` 
        or `filename`

    story_id : str, optional
        The the id of the story in the Presalytics API Story service.  If not provided, 
        a new story will be created.  Do not supply if this object has not yet been created. 
    
    object_ooxml_id : str, optional
        The identifier of the Ooxml Automation service object bound the Story. Do not supply if this 
        object has not yet been created.

    endpoint_map : presalytics.lib.widgets.ooxml.OoxmlEndpointMap, optional
        Reference to the Presalytics API Ooxml Automation service endpoint and object type
        that for the object of interest

    object_name : str, optional
        The name of the object in the file's object tree the will be rendered

    previous_ooxml_version : str, optional
        The id Ooxml Automation service document object that was previously used to 
        occupy this widget in the `presalytics.story.outline.StoryOutline`

    file_last_modified : str, optional
        The "last modified" date for the file at the `filename` path.  Used to ascertain
        whether file has been updated since the last time the widget was initialized, and
        correspondingly, whether the widget should be updated in the `presalytics.story.outline.StoryOutline`

    document_ooxml_id : str, optional
        The identifier for the parent "Document" object in the Ooxml Automation service for the object
        idenitifiable by a combinatation of `object_ooxml_id` and `endpoint_map`.
    
    """
    object_name: typing.Optional[str]
    ooxml_id: str
    file_last_modified: datetime.datetime
    previous_ooxml_version: typing.Dict[str, str]

    __component_kind__ = 'ooxml-file-object'

    def __init__(self,
                 filename,
                 name=None,
                 story_id=None,
                 object_ooxml_id=None,
                 endpoint_map=None,
                 object_name=None,
                 previous_ooxml_version={},
                 file_last_modified=None,
                 document_ooxml_id=None,
                 **kwargs):
        if object_name:
            self.object_name = object_name
        else:
            self.object_name = None
        if not name:
            if self.object_name:
                name = self.object_name
            else:
                name = filename
        super(OoxmlFileWidget, self).__init__(name, story_id, object_ooxml_id, endpoint_map, **kwargs)
        self.filename = os.path.basename(filename)
        if not self.endpoint_map:
            if filename.split(".")[-1] in ["pptx", "ppt"]:
                self.endpoint_map = OoxmlEndpointMap(OoxmlEndpointMap.SLIDE)
        self.previous_ooxml_version = previous_ooxml_version
        if file_last_modified:
            self.file_last_modified = file_last_modified.replace(tzinfo=datetime.timezone.utc)
        else:
            self.file_last_modified = datetime.datetime.utcnow()
        self.document_ooxml_id = document_ooxml_id
        self.update()
        self.svg_html = self.create_container(**kwargs)

    def update(self):
        """
        If the file is available locally, this renders that updated file and pushes
        the updated rendering data to the server
        """
        story: 'ApiStory'
        page: 'Page'
        widget: 'Widget'

        search_paths = list(set(presalytics.autodiscover_paths))
        if os.getcwd() not in search_paths:
            search_paths.append(os.getcwd())
        for path in search_paths:
            fpath = os.path.join(path, self.filename)
            if os.path.exists(fpath):
                # update only the file has been modified sine last time
                this_file_last_modified = datetime.datetime.utcfromtimestamp(os.path.getmtime(fpath)).astimezone(tz=datetime.timezone.utc)
                if self.file_last_modified is None or self.file_last_modified <= this_file_last_modified:
                    client = self.get_client()
                    story, status, headers = client.story.story_id_file_post_with_http_info(self.story_id, file=fpath, replace_existing=True, obsolete_id=self.document_ooxml_id)
                    if status >= 299:
                        raise presalytics.lib.exceptions.ApiError()
                    self.previous_ooxml_version = {
                        "document_ooxml_id": self.document_ooxml_id,
                        "object_ooxml_id": self.object_ooxml_id
                    }
                    new_outline = presalytics.story.outline.StoryOutline.load(story.outline)
                    for page in new_outline.pages:
                        found = False
                        for widget in page.widgets:
                            if widget.name == self.object_name:
                                self.document_ooxml_id = widget.data["document_ooxml_id"]
                                found = True
                                break
                        if found:
                            break
                    if not found:
                        message = "Unable to find widget object name {0} in new story outline.  Has this widget been deleted?".format(self.object_name)
                        raise presalytics.lib.exceptions.ValidationError(message)

                    # Get object tree, compare to object name
                    child_tree = client.ooxml_automation.documents_childobjects_get_id(self.document_ooxml_id)
                    target_dto = None

                    try:
                        target_dto = next(x for x in child_tree if x.entity_name == self.object_name)
                    except StopIteration:
                        pass
                    # if name not in object or _object_name is none, get first item of type in end_point id
                    if not target_dto:
                        try:
                            target_dto = next(x for x in child_tree if x.object_type.split(".")[1] == self.endpoint_map.endpoint_id)
                        except StopIteration:
                            message = "Child tree of document {0} does not have a child object of type {1} or name {2}.".format(self.ooxml_id, self.endpoint_map.endpoint_id, self.object_name)
                            raise presalytics.lib.exceptions.InvalidConfigurationError(message)
                    # set widget parameters for recreation server-side (without file)
                    self.object_ooxml_id = target_dto.entity_id
                    self.file_last_modified = presalytics.lib.util.roundup_date_modified(this_file_last_modified)


    @classmethod
    def deserialize(cls, component, **kwargs):
        init_args = {
            "filename": component.data["filename"],
            "endpoint_map": OoxmlEndpointMap(component.data["endpoint_id"]),
            "object_name": component.data["object_name"],
            "name": component.name,
        }
        if "document_ooxml_id" in component.data:
            init_args.update(
                {
                    "document_ooxml_id": component.data["document_ooxml_id"]
                }
            )
        if "object_ooxml_id" in component.data:
            init_args.update(
                {
                    "object_ooxml_id": component.data["object_ooxml_id"]
                }
            )
        if "file_last_modified" in component.data:
            init_args.update(
                {
                    "file_last_modified": dateutil.parser.parse(component.data["file_last_modified"]).replace(tzinfo=datetime.timezone.utc)
                }
            )
        if "previous_ooxml_version" in component.data:
            init_args.update(
                {
                    "previous_ooxml_version": component.data["previous_ooxml_version"]
                }
            )
        if "story_id" in component.data:
            init_args.update(
                {
                    "story_id": component.data["story_id"]
                }
            )
        if len(kwargs.keys()) > 0:
            init_args.update(kwargs)
        return cls(**init_args)

    def serialize(self):
        self.update()
        data = {
            "filename": self.filename,
            "object_name": self.object_name,
            "endpoint_id": self.endpoint_map.endpoint_id,
            "document_ooxml_id": self.document_ooxml_id,
            "object_ooxml_id": self.object_ooxml_id,
            "story_id": self.story_id
        }
        if self.file_last_modified:
            data.update(
                {
                    "file_last_modified": self.file_last_modified.isoformat()
                }
            )
        if self.previous_ooxml_version:
            data.update(
                {
                    "previous_ooxml_version": self.previous_ooxml_version
                }
            )
        widget = presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
            plugins=None
        )
        return widget


class UpdaterWidgetBase(OoxmlWidgetBase):
    """
    Abstract class for create simple interfaces to update widgets from a simple data table. 

    This class simplifies updates to Ooxml Automation service endpoints, allowing updates to 
    ooxml object data and its underlying xml via simple data transfer objects definted in the 
    Presalytics Ooxml Automation server.  

    Inheriting classes must override the `_get_dto_class`, `_get_endpoint_path`, and `_get_dto_table_name`
    methods

    Parameters
    ----------
    name : str
        The widget name.  If not provided, will be the `object_name` or `filename`

    story_id : str
        The the id of the story in the Presalytics API Story service.  If not provided, 
        a new story will be created.  Do not supply if this object has not yet been created. 
    
    object_id : str
        The identifier of the Ooxml Automation service object bound the Story. Do not supply if this 
        object has not yet been created.

    endpoint_map : presalytics.lib.widgets.ooxml.OoxmlEndpointMap
        Reference to the Presalytics API Ooxml Automation service endpoint and object type
        that for the object of interest

    dto: object, optional
        A an instance of the data transfer object model. The class of this object is defined by the
        `_get_dto_class()` method.  Represents the current state of the data of the data in the service.

    data_table: list of lists, optional
        A representation of a data table that will be incorporates into a dto object's property defined by
        the subclass' `_get_dto_table_name` method.

    """
    def __init__(self, 
                name,
                story_id: str,
                object_id: str,
                endpoint_map: OoxmlEndpointMap,
                dto=None, 
                data_table=None,
                **kwargs):
        super(UpdaterWidgetBase, self).__init__(name, story_id, object_id, endpoint_map, **kwargs)
        self.dto = dto
        self.data_table = data_table
        self.object_id = object_id
    
    
    @abc.abstractmethod
    def _get_dto_class(self) -> typing.Type:
        """
        Returns the Class of instances that will used for dtos
        """
        return NotImplemented

    @abc.abstractmethod
    def _get_endpoint_path(self) -> str:
        """
        Returns the relative path to the the endpoint used , starting from the `root_url` of the 
        `presalytics.lib.widgets.ooxml.OoxmlEndpointMap` object. 
        """
        return NotImplemented

    @abc.abstractmethod
    def _get_dto_table_name(self) -> str:
        """
        Returns the attribute name in the object returned by `_get_dto_class()` that the `data_table` parameter
        should be mapped to.
        """
        return NotImplemented

    def build_endpoint(self) -> str:
        """
        Returns the endpoints used for Api calls
        """
        return posixpath.join(self.endpoint_map.root_url, self._get_endpoint_path(), self.object_ooxml_id)
    
    def get_dto(self):
        """
        Returns an instance of the dto object from the OoxmlAutomation Service API
        """
        client = self.get_client()
        headers = client.get_auth_header()
        headers.update(client.get_request_id_header())
        resp = requests.get(self.build_endpoint(), headers=headers)
        if resp.status_code > 299:
            raise presalytics.lib.exceptions.ApiError(message=resp.text)
        dto = client.ooxml_automation.api_client._ApiClient__deserialize(resp.json(), self._get_dto_class())
        self.dto = dto
        return dto

    def _put_dto(self, dto):
        """
        Updates the Endpoint in the Ooxml Automation Service from a dto object
        """
        client = self.get_client()
        headers = client.get_auth_header()
        headers.update(client.get_request_id_header())
        data = client.ooxml_automation.api_client.sanitize_for_serialization(dto)
        resp = requests.put(self.build_endpoint(), json=data, headers=headers)
        if resp.status_code > 299:
            raise presalytics.lib.exceptions.ApiError(message=resp.text)
        else:
            self.dto = dto

    def update_from_dto(self, dto):
        """
        Updates the Endpoint in the Ooxml Automation Service from a dto object, and updates
        the `svg_html` attribute
        """
        self._put_dto(dto)
        self.svg_html = self.create_container()

    def update_from_data_table(self, data_table):
        """
        Updates the Endpoint in the Ooxml Automation Service from a the `data_table`, and updates
        the `svg_html` attribute
        """
        dto = self.get_dto()
        setattr(dto, self._get_dto_table_name(), data_table)
        self.put_dto(dto)

    def serialize(self):
        data = {
            "story_id": self.story_id,
            "object_id": self.object_ooxml_id,
            "dto": self.dto.to_dict(),
            "data_table": self.data_table
        }
        widget = presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
            plugins=None
        )
        return widget

    @classmethod
    def deserialize(cls, component, **kwargs):
        return cls(
            component.name, 
            component.data["story_id"], 
            component.data["object_id"],
            component.data.get("dto", None),
            component.data.get("data_table", None),
            **kwargs
        )

        
class ChartUpdaterWidget(UpdaterWidgetBase):
    """
    Updates a Chart in the Ooxml Automation service API at the the endpoint '/Chart/ChartUpdate/'

    This class simplifies chart updates, for charts residing in the Ooxml Automation Service, 
    allowing updates to ooxml object data and its underlying xml either via a list of lists or
    the `presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_dto.ChartChartDataDTO`
    object.  

    Parameters
    ----------
    name : str
        A name for the widget.

    story_id : str
        The the id of the story in the Presalytics API Story service. 
    
    chart_id : str
        The identifier of the Ooxml Automation Chart service object. 

    dto: presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_dto.ChartChartDataDTO, optional
        A an instance of the data transfer object model. The class of this object is defined by the
        `_get_dto_class()` method.  Represents the current state of the data of the data in the service.

    data_table: list of lists, optional
        A representation of a data table that will be incorporates into a dto object's property defined by
        the subclass' `_get_dto_table_name` method.

    """
    __component_kind__ = "chart-updater"
    
    def __init__(self, 
                name,
                story_id: str,
                chart_id: str,
                dto: 'ChartChartDataDTO' = None, 
                data_table: typing.Sequence[typing.Sequence] = None,
                **kwargs):
        super().__init__(name, story_id, chart_id, OoxmlEndpointMap.chart(), dto=dto, data_table=data_table, **kwargs)
        self.chart_id = chart_id


    def _get_dto_class(self):
        return presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_dto.ChartChartDataDTO

    def _get_endpoint_path(self):
        return "ChartUpdate"

    def _get_dto_table_name(self):
        return "data_points"

    def get_dataframe(self) -> pandas.DataFrame:
        """
        Returns a panda datagrame of the 
        """
        data: collections.OrderedDict
        
        if not self.dto:
            self.dto = self.get_dto()
        data = collections.OrderedDict()
        for i in range(0, len(self.dto.series_names)):
            data.update({
                self.dto.series_names[i]: pandas.Series(self.dto.data_points[i], self.dto.category_names)
            })
        return pandas.DataFrame(data)
    
    def put_dataframe(self, df: pandas.DataFrame):
        data_dict = df.to_dict('split')
        data_points = list(map(list, zip(*data_dict['data'])))
        dto = self._get_dto_class()(chart_id=self.chart_id, 
                                 series_names=data_dict["columns"], 
                                 category_names=data_dict["index"],
                                 data_points=data_points)
        self._put_dto(dto)

        

class TableUpdaterWidget(UpdaterWidgetBase):
    """
    Updates a Table in the Ooxml Automation service API at the the endpoint '/Table/TableUpdate/'

    This class simplifies table updates, for tables residing in the Ooxml Automation Service, 
    allowing updates to ooxml object data and its underlying xml either via a list of lists or
    the `presalytics.client.presalytics_ooxml_automation.models.table_table_data_dto.TableTableDataDTO`
    object.  

    Parameters
    ----------
    name : str
        A name for the widget.

    story_id : str
        The the id of the story in the Presalytics API Story service. 
    
    table_id : str
        The identifier of the Ooxml Automation Table service object. 

    dto: presalytics.client.presalytics_ooxml_automation.models.table_table_data_dto.TableTableDataDTO, optional
        A an instance of the data transfer object model. The class of this object is defined by the
        `_get_dto_class()` method.  Represents the current state of the data of the data in the service.

    data_table: list of lists, optional
        A representation of a data table that will be incorporates into a dto object's property defined by
        the subclass' `_get_dto_table_name` method.

    """
    __component_kind__ = "table-updater"
    
    def __init__(self, 
                name,
                story_id: str,
                table_id: str,
                dto: 'TableTableDataDTO' = None, 
                data_table: typing.Sequence[typing.Sequence] = None,
                **kwargs):
        super().__init__(name, story_id, table_id, OoxmlEndpointMap.table(), dto=dto, data_table=data_table, **kwargs)
        self.table_id = table_id


    def _get_dto_class(self):
        return presalytics.client.presalytics_ooxml_automation.models.table_table_data_dto.TableTableDataDTO

    def _get_endpoint_path(self):
        return "TableUpdate"

    def _get_dto_table_name(self):
        return "table_data"
    


# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SlideGroupElements(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'shape_tree_id': 'str',
        'parent_group_element_id': 'str',
        'group_element_type_id': 'int',
        'group_element_type_pk': 'str',
        'ultimate_parent_shape_tree_id': 'str',
        'id': 'str'
    }

    attribute_map = {
        'shape_tree_id': 'shapeTreeId',
        'parent_group_element_id': 'parentGroupElementId',
        'group_element_type_id': 'groupElementTypeId',
        'group_element_type_pk': 'groupElementTypePk',
        'ultimate_parent_shape_tree_id': 'ultimateParentShapeTreeId',
        'id': 'id'
    }

    def __init__(self, shape_tree_id=None, parent_group_element_id=None, group_element_type_id=None, group_element_type_pk=None, ultimate_parent_shape_tree_id=None, id=None):  # noqa: E501
        """SlideGroupElements - a model defined in OpenAPI"""  # noqa: E501

        self._shape_tree_id = None
        self._parent_group_element_id = None
        self._group_element_type_id = None
        self._group_element_type_pk = None
        self._ultimate_parent_shape_tree_id = None
        self._id = None
        self.discriminator = None

        self.shape_tree_id = shape_tree_id
        self.parent_group_element_id = parent_group_element_id
        if group_element_type_id is not None:
            self.group_element_type_id = group_element_type_id
        self.group_element_type_pk = group_element_type_pk
        self.ultimate_parent_shape_tree_id = ultimate_parent_shape_tree_id
        if id is not None:
            self.id = id

    @property
    def shape_tree_id(self):
        """Gets the shape_tree_id of this SlideGroupElements.  # noqa: E501


        :return: The shape_tree_id of this SlideGroupElements.  # noqa: E501
        :rtype: str
        """
        return self._shape_tree_id

    @shape_tree_id.setter
    def shape_tree_id(self, shape_tree_id):
        """Sets the shape_tree_id of this SlideGroupElements.


        :param shape_tree_id: The shape_tree_id of this SlideGroupElements.  # noqa: E501
        :type: str
        """

        self._shape_tree_id = shape_tree_id

    @property
    def parent_group_element_id(self):
        """Gets the parent_group_element_id of this SlideGroupElements.  # noqa: E501


        :return: The parent_group_element_id of this SlideGroupElements.  # noqa: E501
        :rtype: str
        """
        return self._parent_group_element_id

    @parent_group_element_id.setter
    def parent_group_element_id(self, parent_group_element_id):
        """Sets the parent_group_element_id of this SlideGroupElements.


        :param parent_group_element_id: The parent_group_element_id of this SlideGroupElements.  # noqa: E501
        :type: str
        """

        self._parent_group_element_id = parent_group_element_id

    @property
    def group_element_type_id(self):
        """Gets the group_element_type_id of this SlideGroupElements.  # noqa: E501


        :return: The group_element_type_id of this SlideGroupElements.  # noqa: E501
        :rtype: int
        """
        return self._group_element_type_id

    @group_element_type_id.setter
    def group_element_type_id(self, group_element_type_id):
        """Sets the group_element_type_id of this SlideGroupElements.


        :param group_element_type_id: The group_element_type_id of this SlideGroupElements.  # noqa: E501
        :type: int
        """

        self._group_element_type_id = group_element_type_id

    @property
    def group_element_type_pk(self):
        """Gets the group_element_type_pk of this SlideGroupElements.  # noqa: E501


        :return: The group_element_type_pk of this SlideGroupElements.  # noqa: E501
        :rtype: str
        """
        return self._group_element_type_pk

    @group_element_type_pk.setter
    def group_element_type_pk(self, group_element_type_pk):
        """Sets the group_element_type_pk of this SlideGroupElements.


        :param group_element_type_pk: The group_element_type_pk of this SlideGroupElements.  # noqa: E501
        :type: str
        """

        self._group_element_type_pk = group_element_type_pk

    @property
    def ultimate_parent_shape_tree_id(self):
        """Gets the ultimate_parent_shape_tree_id of this SlideGroupElements.  # noqa: E501


        :return: The ultimate_parent_shape_tree_id of this SlideGroupElements.  # noqa: E501
        :rtype: str
        """
        return self._ultimate_parent_shape_tree_id

    @ultimate_parent_shape_tree_id.setter
    def ultimate_parent_shape_tree_id(self, ultimate_parent_shape_tree_id):
        """Sets the ultimate_parent_shape_tree_id of this SlideGroupElements.


        :param ultimate_parent_shape_tree_id: The ultimate_parent_shape_tree_id of this SlideGroupElements.  # noqa: E501
        :type: str
        """

        self._ultimate_parent_shape_tree_id = ultimate_parent_shape_tree_id

    @property
    def id(self):
        """Gets the id of this SlideGroupElements.  # noqa: E501


        :return: The id of this SlideGroupElements.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideGroupElements.


        :param id: The id of this SlideGroupElements.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideGroupElements):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class Event(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_at': 'datetime',
        'created_by': 'str',
        'id': 'str',
        'updated_at': 'datetime',
        'updated_by': 'str',
        'attributes': 'str',
        'chat_prompt': 'str',
        'conversation_id': 'str',
        'dom_selectors': 'str',
        'is_chat_hidden': 'bool',
        'is_notify_enabled': 'bool',
        'name': 'str'
    }

    attribute_map = {
        'created_at': 'created_at',
        'created_by': 'created_by',
        'id': 'id',
        'updated_at': 'updated_at',
        'updated_by': 'updated_by',
        'attributes': 'attributes',
        'chat_prompt': 'chat_prompt',
        'conversation_id': 'conversation_id',
        'dom_selectors': 'dom_selectors',
        'is_chat_hidden': 'is_chat_hidden',
        'is_notify_enabled': 'is_notify_enabled',
        'name': 'name'
    }

    def __init__(self, created_at=None, created_by=None, id=None, updated_at=None, updated_by=None, attributes=None, chat_prompt=None, conversation_id=None, dom_selectors=None, is_chat_hidden=None, is_notify_enabled=None, name=None):  # noqa: E501
        """Event - a model defined in OpenAPI"""  # noqa: E501

        self._created_at = None
        self._created_by = None
        self._id = None
        self._updated_at = None
        self._updated_by = None
        self._attributes = None
        self._chat_prompt = None
        self._conversation_id = None
        self._dom_selectors = None
        self._is_chat_hidden = None
        self._is_notify_enabled = None
        self._name = None
        self.discriminator = None

        if created_at is not None:
            self.created_at = created_at
        if created_by is not None:
            self.created_by = created_by
        if id is not None:
            self.id = id
        if updated_at is not None:
            self.updated_at = updated_at
        if updated_by is not None:
            self.updated_by = updated_by
        if attributes is not None:
            self.attributes = attributes
        if chat_prompt is not None:
            self.chat_prompt = chat_prompt
        if conversation_id is not None:
            self.conversation_id = conversation_id
        if dom_selectors is not None:
            self.dom_selectors = dom_selectors
        if is_chat_hidden is not None:
            self.is_chat_hidden = is_chat_hidden
        if is_notify_enabled is not None:
            self.is_notify_enabled = is_notify_enabled
        if name is not None:
            self.name = name

    @property
    def created_at(self):
        """Gets the created_at of this Event.  # noqa: E501


        :return: The created_at of this Event.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Event.


        :param created_at: The created_at of this Event.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def created_by(self):
        """Gets the created_by of this Event.  # noqa: E501


        :return: The created_by of this Event.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this Event.


        :param created_by: The created_by of this Event.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def id(self):
        """Gets the id of this Event.  # noqa: E501


        :return: The id of this Event.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Event.


        :param id: The id of this Event.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def updated_at(self):
        """Gets the updated_at of this Event.  # noqa: E501


        :return: The updated_at of this Event.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Event.


        :param updated_at: The updated_at of this Event.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def updated_by(self):
        """Gets the updated_by of this Event.  # noqa: E501


        :return: The updated_by of this Event.  # noqa: E501
        :rtype: str
        """
        return self._updated_by

    @updated_by.setter
    def updated_by(self, updated_by):
        """Sets the updated_by of this Event.


        :param updated_by: The updated_by of this Event.  # noqa: E501
        :type: str
        """

        self._updated_by = updated_by

    @property
    def attributes(self):
        """Gets the attributes of this Event.  # noqa: E501


        :return: The attributes of this Event.  # noqa: E501
        :rtype: str
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this Event.


        :param attributes: The attributes of this Event.  # noqa: E501
        :type: str
        """

        self._attributes = attributes

    @property
    def chat_prompt(self):
        """Gets the chat_prompt of this Event.  # noqa: E501


        :return: The chat_prompt of this Event.  # noqa: E501
        :rtype: str
        """
        return self._chat_prompt

    @chat_prompt.setter
    def chat_prompt(self, chat_prompt):
        """Sets the chat_prompt of this Event.


        :param chat_prompt: The chat_prompt of this Event.  # noqa: E501
        :type: str
        """

        self._chat_prompt = chat_prompt

    @property
    def conversation_id(self):
        """Gets the conversation_id of this Event.  # noqa: E501


        :return: The conversation_id of this Event.  # noqa: E501
        :rtype: str
        """
        return self._conversation_id

    @conversation_id.setter
    def conversation_id(self, conversation_id):
        """Sets the conversation_id of this Event.


        :param conversation_id: The conversation_id of this Event.  # noqa: E501
        :type: str
        """

        self._conversation_id = conversation_id

    @property
    def dom_selectors(self):
        """Gets the dom_selectors of this Event.  # noqa: E501


        :return: The dom_selectors of this Event.  # noqa: E501
        :rtype: str
        """
        return self._dom_selectors

    @dom_selectors.setter
    def dom_selectors(self, dom_selectors):
        """Sets the dom_selectors of this Event.


        :param dom_selectors: The dom_selectors of this Event.  # noqa: E501
        :type: str
        """

        self._dom_selectors = dom_selectors

    @property
    def is_chat_hidden(self):
        """Gets the is_chat_hidden of this Event.  # noqa: E501


        :return: The is_chat_hidden of this Event.  # noqa: E501
        :rtype: bool
        """
        return self._is_chat_hidden

    @is_chat_hidden.setter
    def is_chat_hidden(self, is_chat_hidden):
        """Sets the is_chat_hidden of this Event.


        :param is_chat_hidden: The is_chat_hidden of this Event.  # noqa: E501
        :type: bool
        """

        self._is_chat_hidden = is_chat_hidden

    @property
    def is_notify_enabled(self):
        """Gets the is_notify_enabled of this Event.  # noqa: E501


        :return: The is_notify_enabled of this Event.  # noqa: E501
        :rtype: bool
        """
        return self._is_notify_enabled

    @is_notify_enabled.setter
    def is_notify_enabled(self, is_notify_enabled):
        """Sets the is_notify_enabled of this Event.


        :param is_notify_enabled: The is_notify_enabled of this Event.  # noqa: E501
        :type: bool
        """

        self._is_notify_enabled = is_notify_enabled

    @property
    def name(self):
        """Gets the name of this Event.  # noqa: E501


        :return: The name of this Event.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Event.


        :param name: The name of this Event.  # noqa: E501
        :type: str
        """

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Event):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class PermissionTypeAllOf(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'can_add_collaborators': 'bool',
        'can_delete': 'bool',
        'can_edit': 'bool',
        'can_view': 'bool',
        'name': 'str'
    }

    attribute_map = {
        'can_add_collaborators': 'can_add_collaborators',
        'can_delete': 'can_delete',
        'can_edit': 'can_edit',
        'can_view': 'can_view',
        'name': 'name'
    }

    def __init__(self, can_add_collaborators=None, can_delete=None, can_edit=None, can_view=None, name=None):  # noqa: E501
        """PermissionTypeAllOf - a model defined in OpenAPI"""  # noqa: E501

        self._can_add_collaborators = None
        self._can_delete = None
        self._can_edit = None
        self._can_view = None
        self._name = None
        self.discriminator = None

        if can_add_collaborators is not None:
            self.can_add_collaborators = can_add_collaborators
        if can_delete is not None:
            self.can_delete = can_delete
        if can_edit is not None:
            self.can_edit = can_edit
        if can_view is not None:
            self.can_view = can_view
        if name is not None:
            self.name = name

    @property
    def can_add_collaborators(self):
        """Gets the can_add_collaborators of this PermissionTypeAllOf.  # noqa: E501


        :return: The can_add_collaborators of this PermissionTypeAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._can_add_collaborators

    @can_add_collaborators.setter
    def can_add_collaborators(self, can_add_collaborators):
        """Sets the can_add_collaborators of this PermissionTypeAllOf.


        :param can_add_collaborators: The can_add_collaborators of this PermissionTypeAllOf.  # noqa: E501
        :type: bool
        """

        self._can_add_collaborators = can_add_collaborators

    @property
    def can_delete(self):
        """Gets the can_delete of this PermissionTypeAllOf.  # noqa: E501


        :return: The can_delete of this PermissionTypeAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._can_delete

    @can_delete.setter
    def can_delete(self, can_delete):
        """Sets the can_delete of this PermissionTypeAllOf.


        :param can_delete: The can_delete of this PermissionTypeAllOf.  # noqa: E501
        :type: bool
        """

        self._can_delete = can_delete

    @property
    def can_edit(self):
        """Gets the can_edit of this PermissionTypeAllOf.  # noqa: E501


        :return: The can_edit of this PermissionTypeAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._can_edit

    @can_edit.setter
    def can_edit(self, can_edit):
        """Sets the can_edit of this PermissionTypeAllOf.


        :param can_edit: The can_edit of this PermissionTypeAllOf.  # noqa: E501
        :type: bool
        """

        self._can_edit = can_edit

    @property
    def can_view(self):
        """Gets the can_view of this PermissionTypeAllOf.  # noqa: E501


        :return: The can_view of this PermissionTypeAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._can_view

    @can_view.setter
    def can_view(self, can_view):
        """Sets the can_view of this PermissionTypeAllOf.


        :param can_view: The can_view of this PermissionTypeAllOf.  # noqa: E501
        :type: bool
        """

        self._can_view = can_view

    @property
    def name(self):
        """Gets the name of this PermissionTypeAllOf.  # noqa: E501


        :return: The name of this PermissionTypeAllOf.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PermissionTypeAllOf.


        :param name: The name of this PermissionTypeAllOf.  # noqa: E501
        :type: str
        """

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PermissionTypeAllOf):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import datetime
import presalytics.lib.constants

class classproperty(property):
    def __get__(self, obj, objtype=None):
        return super(classproperty, self).__get__(objtype)

    def __set__(self, obj, value):
        super(classproperty, self).__set__(type(obj), value)

    def __delete__(self, obj):
        super(classproperty, self).__delete__(type(obj))


def roundup_date_modified(current_datetime: datetime.datetime):
    one_second = datetime.timedelta(seconds=1)
    rounddown = current_datetime.replace(microsecond=0)
    return rounddown + one_second
    
def get_site_host():
    site_host = presalytics.lib.constants.SITE_HOST
    try: 
        site_host = presalytics.CONFIG["HOSTS"]["SITE"]
    except (KeyError, AttributeError):
        pass
    return site_host

        



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class DocumentDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'story_id': 'str',
        'filename': 'str',
        'owner_guid': 'str',
        'document_type_id': 'int',
        'blob_location': 'str',
        'table_styles_xml_blob_url': 'str',
        'title': 'str',
        'slides': 'list[SlideSlidesDetails]',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'story_id': 'storyId',
        'filename': 'filename',
        'owner_guid': 'ownerGuid',
        'document_type_id': 'documentTypeId',
        'blob_location': 'blobLocation',
        'table_styles_xml_blob_url': 'tableStylesXmlBlobUrl',
        'title': 'title',
        'slides': 'slides',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, story_id=None, filename=None, owner_guid=None, document_type_id=None, blob_location=None, table_styles_xml_blob_url=None, title=None, slides=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """DocumentDetails - a model defined in OpenAPI"""  # noqa: E501

        self._story_id = None
        self._filename = None
        self._owner_guid = None
        self._document_type_id = None
        self._blob_location = None
        self._table_styles_xml_blob_url = None
        self._title = None
        self._slides = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        if story_id is not None:
            self.story_id = story_id
        self.filename = filename
        if owner_guid is not None:
            self.owner_guid = owner_guid
        if document_type_id is not None:
            self.document_type_id = document_type_id
        self.blob_location = blob_location
        self.table_styles_xml_blob_url = table_styles_xml_blob_url
        self.title = title
        self.slides = slides
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def story_id(self):
        """Gets the story_id of this DocumentDetails.  # noqa: E501


        :return: The story_id of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._story_id

    @story_id.setter
    def story_id(self, story_id):
        """Sets the story_id of this DocumentDetails.


        :param story_id: The story_id of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._story_id = story_id

    @property
    def filename(self):
        """Gets the filename of this DocumentDetails.  # noqa: E501


        :return: The filename of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._filename

    @filename.setter
    def filename(self, filename):
        """Sets the filename of this DocumentDetails.


        :param filename: The filename of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._filename = filename

    @property
    def owner_guid(self):
        """Gets the owner_guid of this DocumentDetails.  # noqa: E501


        :return: The owner_guid of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._owner_guid

    @owner_guid.setter
    def owner_guid(self, owner_guid):
        """Sets the owner_guid of this DocumentDetails.


        :param owner_guid: The owner_guid of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._owner_guid = owner_guid

    @property
    def document_type_id(self):
        """Gets the document_type_id of this DocumentDetails.  # noqa: E501


        :return: The document_type_id of this DocumentDetails.  # noqa: E501
        :rtype: int
        """
        return self._document_type_id

    @document_type_id.setter
    def document_type_id(self, document_type_id):
        """Sets the document_type_id of this DocumentDetails.


        :param document_type_id: The document_type_id of this DocumentDetails.  # noqa: E501
        :type: int
        """

        self._document_type_id = document_type_id

    @property
    def blob_location(self):
        """Gets the blob_location of this DocumentDetails.  # noqa: E501


        :return: The blob_location of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._blob_location

    @blob_location.setter
    def blob_location(self, blob_location):
        """Sets the blob_location of this DocumentDetails.


        :param blob_location: The blob_location of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._blob_location = blob_location

    @property
    def table_styles_xml_blob_url(self):
        """Gets the table_styles_xml_blob_url of this DocumentDetails.  # noqa: E501


        :return: The table_styles_xml_blob_url of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._table_styles_xml_blob_url

    @table_styles_xml_blob_url.setter
    def table_styles_xml_blob_url(self, table_styles_xml_blob_url):
        """Sets the table_styles_xml_blob_url of this DocumentDetails.


        :param table_styles_xml_blob_url: The table_styles_xml_blob_url of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._table_styles_xml_blob_url = table_styles_xml_blob_url

    @property
    def title(self):
        """Gets the title of this DocumentDetails.  # noqa: E501


        :return: The title of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this DocumentDetails.


        :param title: The title of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def slides(self):
        """Gets the slides of this DocumentDetails.  # noqa: E501


        :return: The slides of this DocumentDetails.  # noqa: E501
        :rtype: list[SlideSlidesDetails]
        """
        return self._slides

    @slides.setter
    def slides(self, slides):
        """Sets the slides of this DocumentDetails.


        :param slides: The slides of this DocumentDetails.  # noqa: E501
        :type: list[SlideSlidesDetails]
        """

        self._slides = slides

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this DocumentDetails.  # noqa: E501


        :return: The base_element_blob_url of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this DocumentDetails.


        :param base_element_blob_url: The base_element_blob_url of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this DocumentDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this DocumentDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this DocumentDetails.  # noqa: E501


        :return: The package_uri of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this DocumentDetails.


        :param package_uri: The package_uri of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this DocumentDetails.  # noqa: E501


        :return: The name of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DocumentDetails.


        :param name: The name of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this DocumentDetails.  # noqa: E501


        :return: The id of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this DocumentDetails.


        :param id: The id of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this DocumentDetails.  # noqa: E501


        :return: The date_created of this DocumentDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this DocumentDetails.


        :param date_created: The date_created of this DocumentDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this DocumentDetails.  # noqa: E501


        :return: The user_created of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this DocumentDetails.


        :param user_created: The user_created of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this DocumentDetails.  # noqa: E501


        :return: The date_modified of this DocumentDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this DocumentDetails.


        :param date_modified: The date_modified of this DocumentDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this DocumentDetails.  # noqa: E501


        :return: The user_modified of this DocumentDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this DocumentDetails.


        :param user_modified: The user_modified of this DocumentDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DocumentDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedSolidFillsDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'hex_value': 'str',
        'is_user_color': 'bool',
        'color_type_id': 'int',
        'fill_map_id': 'str',
        'parent_fill_map': 'SharedFillMapDetails',
        'color_transformations': 'SharedColorTransformationsDetails',
        'parent_line_id': 'str',
        'parent_line': 'SharedLinesDetails',
        'parent_text_id': 'str',
        'parent_text': 'SharedTextDetails',
        'parent_gradient_stop_id': 'str',
        'parent_gradient_stop': 'SharedGradientStopsDetails',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'hex_value': 'hexValue',
        'is_user_color': 'isUserColor',
        'color_type_id': 'colorTypeId',
        'fill_map_id': 'fillMapId',
        'parent_fill_map': 'parentFillMap',
        'color_transformations': 'colorTransformations',
        'parent_line_id': 'parentLineId',
        'parent_line': 'parentLine',
        'parent_text_id': 'parentTextId',
        'parent_text': 'parentText',
        'parent_gradient_stop_id': 'parentGradientStopId',
        'parent_gradient_stop': 'parentGradientStop',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, hex_value=None, is_user_color=None, color_type_id=None, fill_map_id=None, parent_fill_map=None, color_transformations=None, parent_line_id=None, parent_line=None, parent_text_id=None, parent_text=None, parent_gradient_stop_id=None, parent_gradient_stop=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SharedSolidFillsDetails - a model defined in OpenAPI"""  # noqa: E501

        self._hex_value = None
        self._is_user_color = None
        self._color_type_id = None
        self._fill_map_id = None
        self._parent_fill_map = None
        self._color_transformations = None
        self._parent_line_id = None
        self._parent_line = None
        self._parent_text_id = None
        self._parent_text = None
        self._parent_gradient_stop_id = None
        self._parent_gradient_stop = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.hex_value = hex_value
        if is_user_color is not None:
            self.is_user_color = is_user_color
        self.color_type_id = color_type_id
        self.fill_map_id = fill_map_id
        if parent_fill_map is not None:
            self.parent_fill_map = parent_fill_map
        if color_transformations is not None:
            self.color_transformations = color_transformations
        self.parent_line_id = parent_line_id
        if parent_line is not None:
            self.parent_line = parent_line
        self.parent_text_id = parent_text_id
        if parent_text is not None:
            self.parent_text = parent_text
        self.parent_gradient_stop_id = parent_gradient_stop_id
        if parent_gradient_stop is not None:
            self.parent_gradient_stop = parent_gradient_stop
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def hex_value(self):
        """Gets the hex_value of this SharedSolidFillsDetails.  # noqa: E501


        :return: The hex_value of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._hex_value

    @hex_value.setter
    def hex_value(self, hex_value):
        """Sets the hex_value of this SharedSolidFillsDetails.


        :param hex_value: The hex_value of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._hex_value = hex_value

    @property
    def is_user_color(self):
        """Gets the is_user_color of this SharedSolidFillsDetails.  # noqa: E501


        :return: The is_user_color of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_user_color

    @is_user_color.setter
    def is_user_color(self, is_user_color):
        """Sets the is_user_color of this SharedSolidFillsDetails.


        :param is_user_color: The is_user_color of this SharedSolidFillsDetails.  # noqa: E501
        :type: bool
        """

        self._is_user_color = is_user_color

    @property
    def color_type_id(self):
        """Gets the color_type_id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The color_type_id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: int
        """
        return self._color_type_id

    @color_type_id.setter
    def color_type_id(self, color_type_id):
        """Sets the color_type_id of this SharedSolidFillsDetails.


        :param color_type_id: The color_type_id of this SharedSolidFillsDetails.  # noqa: E501
        :type: int
        """

        self._color_type_id = color_type_id

    @property
    def fill_map_id(self):
        """Gets the fill_map_id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The fill_map_id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._fill_map_id

    @fill_map_id.setter
    def fill_map_id(self, fill_map_id):
        """Sets the fill_map_id of this SharedSolidFillsDetails.


        :param fill_map_id: The fill_map_id of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._fill_map_id = fill_map_id

    @property
    def parent_fill_map(self):
        """Gets the parent_fill_map of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_fill_map of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: SharedFillMapDetails
        """
        return self._parent_fill_map

    @parent_fill_map.setter
    def parent_fill_map(self, parent_fill_map):
        """Sets the parent_fill_map of this SharedSolidFillsDetails.


        :param parent_fill_map: The parent_fill_map of this SharedSolidFillsDetails.  # noqa: E501
        :type: SharedFillMapDetails
        """

        self._parent_fill_map = parent_fill_map

    @property
    def color_transformations(self):
        """Gets the color_transformations of this SharedSolidFillsDetails.  # noqa: E501


        :return: The color_transformations of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: SharedColorTransformationsDetails
        """
        return self._color_transformations

    @color_transformations.setter
    def color_transformations(self, color_transformations):
        """Sets the color_transformations of this SharedSolidFillsDetails.


        :param color_transformations: The color_transformations of this SharedSolidFillsDetails.  # noqa: E501
        :type: SharedColorTransformationsDetails
        """

        self._color_transformations = color_transformations

    @property
    def parent_line_id(self):
        """Gets the parent_line_id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_line_id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._parent_line_id

    @parent_line_id.setter
    def parent_line_id(self, parent_line_id):
        """Sets the parent_line_id of this SharedSolidFillsDetails.


        :param parent_line_id: The parent_line_id of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._parent_line_id = parent_line_id

    @property
    def parent_line(self):
        """Gets the parent_line of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_line of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: SharedLinesDetails
        """
        return self._parent_line

    @parent_line.setter
    def parent_line(self, parent_line):
        """Sets the parent_line of this SharedSolidFillsDetails.


        :param parent_line: The parent_line of this SharedSolidFillsDetails.  # noqa: E501
        :type: SharedLinesDetails
        """

        self._parent_line = parent_line

    @property
    def parent_text_id(self):
        """Gets the parent_text_id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_text_id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._parent_text_id

    @parent_text_id.setter
    def parent_text_id(self, parent_text_id):
        """Sets the parent_text_id of this SharedSolidFillsDetails.


        :param parent_text_id: The parent_text_id of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._parent_text_id = parent_text_id

    @property
    def parent_text(self):
        """Gets the parent_text of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_text of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: SharedTextDetails
        """
        return self._parent_text

    @parent_text.setter
    def parent_text(self, parent_text):
        """Sets the parent_text of this SharedSolidFillsDetails.


        :param parent_text: The parent_text of this SharedSolidFillsDetails.  # noqa: E501
        :type: SharedTextDetails
        """

        self._parent_text = parent_text

    @property
    def parent_gradient_stop_id(self):
        """Gets the parent_gradient_stop_id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_gradient_stop_id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._parent_gradient_stop_id

    @parent_gradient_stop_id.setter
    def parent_gradient_stop_id(self, parent_gradient_stop_id):
        """Sets the parent_gradient_stop_id of this SharedSolidFillsDetails.


        :param parent_gradient_stop_id: The parent_gradient_stop_id of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._parent_gradient_stop_id = parent_gradient_stop_id

    @property
    def parent_gradient_stop(self):
        """Gets the parent_gradient_stop of this SharedSolidFillsDetails.  # noqa: E501


        :return: The parent_gradient_stop of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: SharedGradientStopsDetails
        """
        return self._parent_gradient_stop

    @parent_gradient_stop.setter
    def parent_gradient_stop(self, parent_gradient_stop):
        """Sets the parent_gradient_stop of this SharedSolidFillsDetails.


        :param parent_gradient_stop: The parent_gradient_stop of this SharedSolidFillsDetails.  # noqa: E501
        :type: SharedGradientStopsDetails
        """

        self._parent_gradient_stop = parent_gradient_stop

    @property
    def id(self):
        """Gets the id of this SharedSolidFillsDetails.  # noqa: E501


        :return: The id of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedSolidFillsDetails.


        :param id: The id of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SharedSolidFillsDetails.  # noqa: E501


        :return: The date_created of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SharedSolidFillsDetails.


        :param date_created: The date_created of this SharedSolidFillsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SharedSolidFillsDetails.  # noqa: E501


        :return: The user_created of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SharedSolidFillsDetails.


        :param user_created: The user_created of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SharedSolidFillsDetails.  # noqa: E501


        :return: The date_modified of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SharedSolidFillsDetails.


        :param date_modified: The date_modified of this SharedSolidFillsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SharedSolidFillsDetails.  # noqa: E501


        :return: The user_modified of this SharedSolidFillsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SharedSolidFillsDetails.


        :param user_modified: The user_modified of this SharedSolidFillsDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedSolidFillsDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
This module conatains objects for interacting with the Presalytics API.  It has three submodules:
`presalytics.client.presalytics_doc_converter`, 'presalytics.client.presalytics_ooxml_automation`, and 
`presalytics.client.presalytics_story` contain that auto-generate code using the 
[OpenAPI Generator](https://github.com/OpenAPITools/openapi-generator) and specifications from 
http://api.presalytics.io .

Modules contained at this level are middlewares to simplify user interaction with the generated 
code base and the Presalytics API.  
 - `presalytics.client.auth` contains authentication and request handling handling middleware
 - `presalytics.client.api` conatains wrapper classes and convenience extensions so that users
only need to instantiate one object to get access to all public microservices in the Presalytics API
"""


# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedLines(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'connector_id': 'str',
        'shape_id': 'str',
        'dash_type_id': 'int',
        'head_end_type_id': 'int',
        'tail_end_type_id': 'int',
        'weight': 'int',
        'head_end_height_id': 'int',
        'head_end_width_id': 'int',
        'tail_end_height_id': 'int',
        'tail_end_width_id': 'int',
        'top_border_id': 'str',
        'bottom_border_id': 'str',
        'right_border_id': 'str',
        'left_border_id': 'str',
        't_lto_br_border_id': 'str',
        'b_lto_tr_border_id': 'str',
        'line_map_id': 'str',
        'id': 'str'
    }

    attribute_map = {
        'connector_id': 'connectorId',
        'shape_id': 'shapeId',
        'dash_type_id': 'dashTypeId',
        'head_end_type_id': 'headEndTypeId',
        'tail_end_type_id': 'tailEndTypeId',
        'weight': 'weight',
        'head_end_height_id': 'headEndHeightId',
        'head_end_width_id': 'headEndWidthId',
        'tail_end_height_id': 'tailEndHeightId',
        'tail_end_width_id': 'tailEndWidthId',
        'top_border_id': 'topBorderId',
        'bottom_border_id': 'bottomBorderId',
        'right_border_id': 'rightBorderId',
        'left_border_id': 'leftBorderId',
        't_lto_br_border_id': 'tLtoBRBorderId',
        'b_lto_tr_border_id': 'bLtoTRBorderId',
        'line_map_id': 'lineMapId',
        'id': 'id'
    }

    def __init__(self, connector_id=None, shape_id=None, dash_type_id=None, head_end_type_id=None, tail_end_type_id=None, weight=None, head_end_height_id=None, head_end_width_id=None, tail_end_height_id=None, tail_end_width_id=None, top_border_id=None, bottom_border_id=None, right_border_id=None, left_border_id=None, t_lto_br_border_id=None, b_lto_tr_border_id=None, line_map_id=None, id=None):  # noqa: E501
        """SharedLines - a model defined in OpenAPI"""  # noqa: E501

        self._connector_id = None
        self._shape_id = None
        self._dash_type_id = None
        self._head_end_type_id = None
        self._tail_end_type_id = None
        self._weight = None
        self._head_end_height_id = None
        self._head_end_width_id = None
        self._tail_end_height_id = None
        self._tail_end_width_id = None
        self._top_border_id = None
        self._bottom_border_id = None
        self._right_border_id = None
        self._left_border_id = None
        self._t_lto_br_border_id = None
        self._b_lto_tr_border_id = None
        self._line_map_id = None
        self._id = None
        self.discriminator = None

        self.connector_id = connector_id
        self.shape_id = shape_id
        if dash_type_id is not None:
            self.dash_type_id = dash_type_id
        if head_end_type_id is not None:
            self.head_end_type_id = head_end_type_id
        if tail_end_type_id is not None:
            self.tail_end_type_id = tail_end_type_id
        if weight is not None:
            self.weight = weight
        if head_end_height_id is not None:
            self.head_end_height_id = head_end_height_id
        if head_end_width_id is not None:
            self.head_end_width_id = head_end_width_id
        if tail_end_height_id is not None:
            self.tail_end_height_id = tail_end_height_id
        if tail_end_width_id is not None:
            self.tail_end_width_id = tail_end_width_id
        self.top_border_id = top_border_id
        self.bottom_border_id = bottom_border_id
        self.right_border_id = right_border_id
        self.left_border_id = left_border_id
        self.t_lto_br_border_id = t_lto_br_border_id
        self.b_lto_tr_border_id = b_lto_tr_border_id
        self.line_map_id = line_map_id
        if id is not None:
            self.id = id

    @property
    def connector_id(self):
        """Gets the connector_id of this SharedLines.  # noqa: E501


        :return: The connector_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._connector_id

    @connector_id.setter
    def connector_id(self, connector_id):
        """Sets the connector_id of this SharedLines.


        :param connector_id: The connector_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._connector_id = connector_id

    @property
    def shape_id(self):
        """Gets the shape_id of this SharedLines.  # noqa: E501


        :return: The shape_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._shape_id

    @shape_id.setter
    def shape_id(self, shape_id):
        """Sets the shape_id of this SharedLines.


        :param shape_id: The shape_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._shape_id = shape_id

    @property
    def dash_type_id(self):
        """Gets the dash_type_id of this SharedLines.  # noqa: E501


        :return: The dash_type_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._dash_type_id

    @dash_type_id.setter
    def dash_type_id(self, dash_type_id):
        """Sets the dash_type_id of this SharedLines.


        :param dash_type_id: The dash_type_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._dash_type_id = dash_type_id

    @property
    def head_end_type_id(self):
        """Gets the head_end_type_id of this SharedLines.  # noqa: E501


        :return: The head_end_type_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._head_end_type_id

    @head_end_type_id.setter
    def head_end_type_id(self, head_end_type_id):
        """Sets the head_end_type_id of this SharedLines.


        :param head_end_type_id: The head_end_type_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._head_end_type_id = head_end_type_id

    @property
    def tail_end_type_id(self):
        """Gets the tail_end_type_id of this SharedLines.  # noqa: E501


        :return: The tail_end_type_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._tail_end_type_id

    @tail_end_type_id.setter
    def tail_end_type_id(self, tail_end_type_id):
        """Sets the tail_end_type_id of this SharedLines.


        :param tail_end_type_id: The tail_end_type_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._tail_end_type_id = tail_end_type_id

    @property
    def weight(self):
        """Gets the weight of this SharedLines.  # noqa: E501


        :return: The weight of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._weight

    @weight.setter
    def weight(self, weight):
        """Sets the weight of this SharedLines.


        :param weight: The weight of this SharedLines.  # noqa: E501
        :type: int
        """

        self._weight = weight

    @property
    def head_end_height_id(self):
        """Gets the head_end_height_id of this SharedLines.  # noqa: E501


        :return: The head_end_height_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._head_end_height_id

    @head_end_height_id.setter
    def head_end_height_id(self, head_end_height_id):
        """Sets the head_end_height_id of this SharedLines.


        :param head_end_height_id: The head_end_height_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._head_end_height_id = head_end_height_id

    @property
    def head_end_width_id(self):
        """Gets the head_end_width_id of this SharedLines.  # noqa: E501


        :return: The head_end_width_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._head_end_width_id

    @head_end_width_id.setter
    def head_end_width_id(self, head_end_width_id):
        """Sets the head_end_width_id of this SharedLines.


        :param head_end_width_id: The head_end_width_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._head_end_width_id = head_end_width_id

    @property
    def tail_end_height_id(self):
        """Gets the tail_end_height_id of this SharedLines.  # noqa: E501


        :return: The tail_end_height_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._tail_end_height_id

    @tail_end_height_id.setter
    def tail_end_height_id(self, tail_end_height_id):
        """Sets the tail_end_height_id of this SharedLines.


        :param tail_end_height_id: The tail_end_height_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._tail_end_height_id = tail_end_height_id

    @property
    def tail_end_width_id(self):
        """Gets the tail_end_width_id of this SharedLines.  # noqa: E501


        :return: The tail_end_width_id of this SharedLines.  # noqa: E501
        :rtype: int
        """
        return self._tail_end_width_id

    @tail_end_width_id.setter
    def tail_end_width_id(self, tail_end_width_id):
        """Sets the tail_end_width_id of this SharedLines.


        :param tail_end_width_id: The tail_end_width_id of this SharedLines.  # noqa: E501
        :type: int
        """

        self._tail_end_width_id = tail_end_width_id

    @property
    def top_border_id(self):
        """Gets the top_border_id of this SharedLines.  # noqa: E501


        :return: The top_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._top_border_id

    @top_border_id.setter
    def top_border_id(self, top_border_id):
        """Sets the top_border_id of this SharedLines.


        :param top_border_id: The top_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._top_border_id = top_border_id

    @property
    def bottom_border_id(self):
        """Gets the bottom_border_id of this SharedLines.  # noqa: E501


        :return: The bottom_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._bottom_border_id

    @bottom_border_id.setter
    def bottom_border_id(self, bottom_border_id):
        """Sets the bottom_border_id of this SharedLines.


        :param bottom_border_id: The bottom_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._bottom_border_id = bottom_border_id

    @property
    def right_border_id(self):
        """Gets the right_border_id of this SharedLines.  # noqa: E501


        :return: The right_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._right_border_id

    @right_border_id.setter
    def right_border_id(self, right_border_id):
        """Sets the right_border_id of this SharedLines.


        :param right_border_id: The right_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._right_border_id = right_border_id

    @property
    def left_border_id(self):
        """Gets the left_border_id of this SharedLines.  # noqa: E501


        :return: The left_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._left_border_id

    @left_border_id.setter
    def left_border_id(self, left_border_id):
        """Sets the left_border_id of this SharedLines.


        :param left_border_id: The left_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._left_border_id = left_border_id

    @property
    def t_lto_br_border_id(self):
        """Gets the t_lto_br_border_id of this SharedLines.  # noqa: E501


        :return: The t_lto_br_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._t_lto_br_border_id

    @t_lto_br_border_id.setter
    def t_lto_br_border_id(self, t_lto_br_border_id):
        """Sets the t_lto_br_border_id of this SharedLines.


        :param t_lto_br_border_id: The t_lto_br_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._t_lto_br_border_id = t_lto_br_border_id

    @property
    def b_lto_tr_border_id(self):
        """Gets the b_lto_tr_border_id of this SharedLines.  # noqa: E501


        :return: The b_lto_tr_border_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._b_lto_tr_border_id

    @b_lto_tr_border_id.setter
    def b_lto_tr_border_id(self, b_lto_tr_border_id):
        """Sets the b_lto_tr_border_id of this SharedLines.


        :param b_lto_tr_border_id: The b_lto_tr_border_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._b_lto_tr_border_id = b_lto_tr_border_id

    @property
    def line_map_id(self):
        """Gets the line_map_id of this SharedLines.  # noqa: E501


        :return: The line_map_id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._line_map_id

    @line_map_id.setter
    def line_map_id(self, line_map_id):
        """Sets the line_map_id of this SharedLines.


        :param line_map_id: The line_map_id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._line_map_id = line_map_id

    @property
    def id(self):
        """Gets the id of this SharedLines.  # noqa: E501


        :return: The id of this SharedLines.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedLines.


        :param id: The id of this SharedLines.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedLines):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class ThemeThemesDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'slide_id': 'str',
        'slide': 'SlideSlidesDetails',
        'background_fills': 'list[ThemeBackgroundFillsDetails]',
        'colors': 'ThemeColorsDetails',
        'custom_colors': 'list[ThemeCustomColorsDetails]',
        'effect_maps': 'list[ThemeEffectMapDetails]',
        'fills': 'list[ThemeFillsDetails]',
        'fonts': 'ThemeFontsDetails',
        'line_maps': 'list[ThemeLineMapDetails]',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'slide_id': 'slideId',
        'slide': 'slide',
        'background_fills': 'backgroundFills',
        'colors': 'colors',
        'custom_colors': 'customColors',
        'effect_maps': 'effectMaps',
        'fills': 'fills',
        'fonts': 'fonts',
        'line_maps': 'lineMaps',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, slide_id=None, slide=None, background_fills=None, colors=None, custom_colors=None, effect_maps=None, fills=None, fonts=None, line_maps=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """ThemeThemesDetails - a model defined in OpenAPI"""  # noqa: E501

        self._slide_id = None
        self._slide = None
        self._background_fills = None
        self._colors = None
        self._custom_colors = None
        self._effect_maps = None
        self._fills = None
        self._fonts = None
        self._line_maps = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.slide_id = slide_id
        if slide is not None:
            self.slide = slide
        self.background_fills = background_fills
        if colors is not None:
            self.colors = colors
        self.custom_colors = custom_colors
        self.effect_maps = effect_maps
        self.fills = fills
        if fonts is not None:
            self.fonts = fonts
        self.line_maps = line_maps
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def slide_id(self):
        """Gets the slide_id of this ThemeThemesDetails.  # noqa: E501


        :return: The slide_id of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._slide_id

    @slide_id.setter
    def slide_id(self, slide_id):
        """Sets the slide_id of this ThemeThemesDetails.


        :param slide_id: The slide_id of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._slide_id = slide_id

    @property
    def slide(self):
        """Gets the slide of this ThemeThemesDetails.  # noqa: E501


        :return: The slide of this ThemeThemesDetails.  # noqa: E501
        :rtype: SlideSlidesDetails
        """
        return self._slide

    @slide.setter
    def slide(self, slide):
        """Sets the slide of this ThemeThemesDetails.


        :param slide: The slide of this ThemeThemesDetails.  # noqa: E501
        :type: SlideSlidesDetails
        """

        self._slide = slide

    @property
    def background_fills(self):
        """Gets the background_fills of this ThemeThemesDetails.  # noqa: E501


        :return: The background_fills of this ThemeThemesDetails.  # noqa: E501
        :rtype: list[ThemeBackgroundFillsDetails]
        """
        return self._background_fills

    @background_fills.setter
    def background_fills(self, background_fills):
        """Sets the background_fills of this ThemeThemesDetails.


        :param background_fills: The background_fills of this ThemeThemesDetails.  # noqa: E501
        :type: list[ThemeBackgroundFillsDetails]
        """

        self._background_fills = background_fills

    @property
    def colors(self):
        """Gets the colors of this ThemeThemesDetails.  # noqa: E501


        :return: The colors of this ThemeThemesDetails.  # noqa: E501
        :rtype: ThemeColorsDetails
        """
        return self._colors

    @colors.setter
    def colors(self, colors):
        """Sets the colors of this ThemeThemesDetails.


        :param colors: The colors of this ThemeThemesDetails.  # noqa: E501
        :type: ThemeColorsDetails
        """

        self._colors = colors

    @property
    def custom_colors(self):
        """Gets the custom_colors of this ThemeThemesDetails.  # noqa: E501


        :return: The custom_colors of this ThemeThemesDetails.  # noqa: E501
        :rtype: list[ThemeCustomColorsDetails]
        """
        return self._custom_colors

    @custom_colors.setter
    def custom_colors(self, custom_colors):
        """Sets the custom_colors of this ThemeThemesDetails.


        :param custom_colors: The custom_colors of this ThemeThemesDetails.  # noqa: E501
        :type: list[ThemeCustomColorsDetails]
        """

        self._custom_colors = custom_colors

    @property
    def effect_maps(self):
        """Gets the effect_maps of this ThemeThemesDetails.  # noqa: E501


        :return: The effect_maps of this ThemeThemesDetails.  # noqa: E501
        :rtype: list[ThemeEffectMapDetails]
        """
        return self._effect_maps

    @effect_maps.setter
    def effect_maps(self, effect_maps):
        """Sets the effect_maps of this ThemeThemesDetails.


        :param effect_maps: The effect_maps of this ThemeThemesDetails.  # noqa: E501
        :type: list[ThemeEffectMapDetails]
        """

        self._effect_maps = effect_maps

    @property
    def fills(self):
        """Gets the fills of this ThemeThemesDetails.  # noqa: E501


        :return: The fills of this ThemeThemesDetails.  # noqa: E501
        :rtype: list[ThemeFillsDetails]
        """
        return self._fills

    @fills.setter
    def fills(self, fills):
        """Sets the fills of this ThemeThemesDetails.


        :param fills: The fills of this ThemeThemesDetails.  # noqa: E501
        :type: list[ThemeFillsDetails]
        """

        self._fills = fills

    @property
    def fonts(self):
        """Gets the fonts of this ThemeThemesDetails.  # noqa: E501


        :return: The fonts of this ThemeThemesDetails.  # noqa: E501
        :rtype: ThemeFontsDetails
        """
        return self._fonts

    @fonts.setter
    def fonts(self, fonts):
        """Sets the fonts of this ThemeThemesDetails.


        :param fonts: The fonts of this ThemeThemesDetails.  # noqa: E501
        :type: ThemeFontsDetails
        """

        self._fonts = fonts

    @property
    def line_maps(self):
        """Gets the line_maps of this ThemeThemesDetails.  # noqa: E501


        :return: The line_maps of this ThemeThemesDetails.  # noqa: E501
        :rtype: list[ThemeLineMapDetails]
        """
        return self._line_maps

    @line_maps.setter
    def line_maps(self, line_maps):
        """Sets the line_maps of this ThemeThemesDetails.


        :param line_maps: The line_maps of this ThemeThemesDetails.  # noqa: E501
        :type: list[ThemeLineMapDetails]
        """

        self._line_maps = line_maps

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this ThemeThemesDetails.  # noqa: E501


        :return: The base_element_blob_url of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this ThemeThemesDetails.


        :param base_element_blob_url: The base_element_blob_url of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this ThemeThemesDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this ThemeThemesDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this ThemeThemesDetails.  # noqa: E501


        :return: The package_uri of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this ThemeThemesDetails.


        :param package_uri: The package_uri of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this ThemeThemesDetails.  # noqa: E501


        :return: The name of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ThemeThemesDetails.


        :param name: The name of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this ThemeThemesDetails.  # noqa: E501


        :return: The id of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ThemeThemesDetails.


        :param id: The id of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this ThemeThemesDetails.  # noqa: E501


        :return: The date_created of this ThemeThemesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this ThemeThemesDetails.


        :param date_created: The date_created of this ThemeThemesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this ThemeThemesDetails.  # noqa: E501


        :return: The user_created of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this ThemeThemesDetails.


        :param user_created: The user_created of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this ThemeThemesDetails.  # noqa: E501


        :return: The date_modified of this ThemeThemesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this ThemeThemesDetails.


        :param date_modified: The date_modified of this ThemeThemesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this ThemeThemesDetails.  # noqa: E501


        :return: The user_modified of this ThemeThemesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this ThemeThemesDetails.


        :param user_modified: The user_modified of this ThemeThemesDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ThemeThemesDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import typing
import presalytics.lib.plugins.base
import presalytics.lib.exceptions

css_path = os.path.join(os.path.dirname(__file__), "css")

class LocalStylesPlugin(presalytics.lib.plugins.base.StylePlugin):
    """
    Plugin incorporate styles from a css stylesheet in a local filepath
    """
    __plugin_name__ = 'local'

    LOCAL_STYLES_MAP = {
        "single_item_grid": os.path.join(css_path, "single-item-grid.css"),
        "flex_row": os.path.join(css_path, "flex-row.css"),
        "light_grey": os.path.join(css_path, "light-grey.css"),
        "responsive_title": os.path.join(css_path, "responsive-title.css"),
        "reveal_overrides": os.path.join(css_path, "reveal-overrides-base.css")
    }
    """
    Dictionary containing a map configuration keys to css files that are 
    member of the Presalytics Python Library manifest
    """

    def to_style(self, config, **kwargs):
        """
        Renders a css stylesheet as an inline style tag in a story

        Parameters
        ----------
        config : dict
            A dictionary containing either a "css_file_path" key, which points
            to a local file path, or a "css_file_id" refers to a css file included
            in the Presalytics Python Library's manifest 
        """
        id = config.get("css_file_id", None)
        if not id:
            file_path = config.get("css_file_path", None)
            if not file_path:
                message = 'Plugin requires "css_file_id" or "css_file_path" in its configuration dictionary'
                raise presalytics.lib.exceptions.MissingConfigException(message)
            if not os.path.exists(file_path):
                message = 'Path {0} in plugin configuration does not exist'.format(file_path)
                raise presalytics.lib.exceptions.MissingConfigException(message)
        else:
            file_path = LocalStylesPlugin.LOCAL_STYLES_MAP[id]
        with open(file_path, 'r') as f:
            style_data = f.read()
        tag = "<style>{0}</style>".format(style_data)
        return tag



# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class EventAllOf(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'str',
        'chat_prompt': 'str',
        'conversation_id': 'str',
        'dom_selectors': 'str',
        'is_chat_hidden': 'bool',
        'is_notify_enabled': 'bool',
        'name': 'str'
    }

    attribute_map = {
        'attributes': 'attributes',
        'chat_prompt': 'chat_prompt',
        'conversation_id': 'conversation_id',
        'dom_selectors': 'dom_selectors',
        'is_chat_hidden': 'is_chat_hidden',
        'is_notify_enabled': 'is_notify_enabled',
        'name': 'name'
    }

    def __init__(self, attributes=None, chat_prompt=None, conversation_id=None, dom_selectors=None, is_chat_hidden=None, is_notify_enabled=None, name=None):  # noqa: E501
        """EventAllOf - a model defined in OpenAPI"""  # noqa: E501

        self._attributes = None
        self._chat_prompt = None
        self._conversation_id = None
        self._dom_selectors = None
        self._is_chat_hidden = None
        self._is_notify_enabled = None
        self._name = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if chat_prompt is not None:
            self.chat_prompt = chat_prompt
        if conversation_id is not None:
            self.conversation_id = conversation_id
        if dom_selectors is not None:
            self.dom_selectors = dom_selectors
        if is_chat_hidden is not None:
            self.is_chat_hidden = is_chat_hidden
        if is_notify_enabled is not None:
            self.is_notify_enabled = is_notify_enabled
        if name is not None:
            self.name = name

    @property
    def attributes(self):
        """Gets the attributes of this EventAllOf.  # noqa: E501


        :return: The attributes of this EventAllOf.  # noqa: E501
        :rtype: str
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this EventAllOf.


        :param attributes: The attributes of this EventAllOf.  # noqa: E501
        :type: str
        """

        self._attributes = attributes

    @property
    def chat_prompt(self):
        """Gets the chat_prompt of this EventAllOf.  # noqa: E501


        :return: The chat_prompt of this EventAllOf.  # noqa: E501
        :rtype: str
        """
        return self._chat_prompt

    @chat_prompt.setter
    def chat_prompt(self, chat_prompt):
        """Sets the chat_prompt of this EventAllOf.


        :param chat_prompt: The chat_prompt of this EventAllOf.  # noqa: E501
        :type: str
        """

        self._chat_prompt = chat_prompt

    @property
    def conversation_id(self):
        """Gets the conversation_id of this EventAllOf.  # noqa: E501


        :return: The conversation_id of this EventAllOf.  # noqa: E501
        :rtype: str
        """
        return self._conversation_id

    @conversation_id.setter
    def conversation_id(self, conversation_id):
        """Sets the conversation_id of this EventAllOf.


        :param conversation_id: The conversation_id of this EventAllOf.  # noqa: E501
        :type: str
        """

        self._conversation_id = conversation_id

    @property
    def dom_selectors(self):
        """Gets the dom_selectors of this EventAllOf.  # noqa: E501


        :return: The dom_selectors of this EventAllOf.  # noqa: E501
        :rtype: str
        """
        return self._dom_selectors

    @dom_selectors.setter
    def dom_selectors(self, dom_selectors):
        """Sets the dom_selectors of this EventAllOf.


        :param dom_selectors: The dom_selectors of this EventAllOf.  # noqa: E501
        :type: str
        """

        self._dom_selectors = dom_selectors

    @property
    def is_chat_hidden(self):
        """Gets the is_chat_hidden of this EventAllOf.  # noqa: E501


        :return: The is_chat_hidden of this EventAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._is_chat_hidden

    @is_chat_hidden.setter
    def is_chat_hidden(self, is_chat_hidden):
        """Sets the is_chat_hidden of this EventAllOf.


        :param is_chat_hidden: The is_chat_hidden of this EventAllOf.  # noqa: E501
        :type: bool
        """

        self._is_chat_hidden = is_chat_hidden

    @property
    def is_notify_enabled(self):
        """Gets the is_notify_enabled of this EventAllOf.  # noqa: E501


        :return: The is_notify_enabled of this EventAllOf.  # noqa: E501
        :rtype: bool
        """
        return self._is_notify_enabled

    @is_notify_enabled.setter
    def is_notify_enabled(self, is_notify_enabled):
        """Sets the is_notify_enabled of this EventAllOf.


        :param is_notify_enabled: The is_notify_enabled of this EventAllOf.  # noqa: E501
        :type: bool
        """

        self._is_notify_enabled = is_notify_enabled

    @property
    def name(self):
        """Gets the name of this EventAllOf.  # noqa: E501


        :return: The name of this EventAllOf.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this EventAllOf.


        :param name: The name of this EventAllOf.  # noqa: E501
        :type: str
        """

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EventAllOf):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import typing
import jinja2
import presalytics.story.outline
import presalytics.story.components
import presalytics.lib.util as util
if typing.TYPE_CHECKING:
    from presalytics.story.components import WidgetBase
    from presalytics.story.outline import Page


class WidgetPage(presalytics.story.components.PageTemplateBase):
    __component_kind__ = 'widget-page'

    def __init__(self, page: 'Page', **kwargs):
        super(WidgetPage, self).__init__(page, **kwargs)

    @classmethod
    def deserialize(cls, component, **kwargs):
        return cls(component, **kwargs)

    def serialize(self):
        return self.outline_page

    def render(self, **kwargs):
        return self.widgets[0].render()


def htmlize(widget):
    """
    Jinja filter to render a widget to a html string
    """
    html = widget.render(widget)
    try:
        html = html.decode('utf-8')
    except Exception:
        pass
    return html
    


class JinjaTemplateBuilder(presalytics.story.components.PageTemplateBase):
    """
    Base class for building objects that render html from `presalytics.story.outline.Page` 
    objects that implementing the [Jinja2](https://jinja.palletsprojects.com/) 
    python library.

    Instances of this class will look for templates located at file location identified by the
    `__template_file__` attribute.  If not template is found, the content in the `template_string`
    attribute will be used. Templates should call each widget's `to_html()` method in a placeholder
    in order to generate valid html for the page.      

    *About building templates*: Templates are passed a `widgets` attribute and a `widget_index` 
    integer (initialized at 0) as part of the context during rendering,. To render 
    multiple widgets on a page, the following pattern can be used inside of templates to 
    increment through the widgets as the jinja2 rendering engine moves through the template:
        
        {{ widgets[widget_index.next()].render() }}  // renders widget and increments widget_index

    Please also note that if `<script>` tags are included in the template, they will be 
    stripped out downstream by a `presalytics.story.components.Renderer` for security
    reasons.  Scripts included in templates will not make it to the browser.

    Parameters
    ----------
    page : presalytics.story.outline.Page
        The page to be rendered

    Attributes
    ----------
    __template_file__ : str
        The filename to an html file containing a fragment that will be rendered into 
        a page by a `presalytics.story.components.Renderer`
    
    __template_paths__ : list of str
        user-defined filepaths to directories where Jinja2 should look for the `__template_file__`

    __css__ : list of str
        Each str in this list is a key that maps to an entry in the
        `presalytics.lib.plugins.local.LocalStylesPlugin.LOCAL_STYLES_MAP`. Ids matched here 
        load the css files as a dependent plugin.

    template_paths : list of str
        The folders Jinja2 will look in. Includes that html directory adjacent to this `__file__`, 
        appending with the files in the `__template_paths__`

    widgets : list of subclass instances of presalytics.story.components.WidgetBase
        Widget to be rendered into the placeholders in the template identified by `__template_file__`
    
    """
    __css__: typing.Sequence[str]
    __template_file__: str
    __template_paths__: typing.List[str]
    template_paths: typing.List[str]

    __template_paths__ = []
    __css__ = []

    class WidgetIndexer(object):
        """
        A counter class for 

        call `widgetindex.next()` in html templates to move this the widget list
        """
        def __init__(self):
            self._val = 0
        
        def next(self):
            """
            Returns the current widget index and increments the value for the next call 
            """
            cur = self._val
            self._val += 1
            return cur

    def __init__(self, page: 'Page', **kwargs) -> None:
        super().__init__(page, **kwargs)
        pkg_templates = os.path.join(os.path.dirname(__file__), "html")
        self.template_paths = [pkg_templates, os.getcwd()]
        if len(self.__template_paths__) > 0:
            self.template_paths[0:0] = self.__template_paths__
        self.is_template_local = self.check_for_file()
        if self.is_template_local:
            self.template_string = self.read_template_string()
        elif self.outline_page.additional_properties.get("template_string", None):
            self.template_string = self.outline_page.additional_properties.get("template_string")
        else:
            self.template_string = None
        if len(kwargs.keys()) > 0:
            self.outline_page.additional_properties.update(kwargs)

    @classmethod
    def deserialize(cls, component, **kwargs):
        return cls(component, **kwargs)

    @util.classproperty
    def __plugins__(cls):
        plugin_list = []
        for id in cls.__css__:
            new_item = {
                'kind': 'style',
                'name': 'local',
                'config': {
                    'css_file_id': id
                }
            }
            plugin_list.append(new_item)
        return plugin_list

    def serialize(self):
        updated_plugins = []
        for plugin_data in self.__plugins__:
            updated_plugins.append(presalytics.story.outline.Plugin(**plugin_data))
        self.outline_page.plugins = updated_plugins
        if self.template_string:
            self.outline_page.additional_properties["template_string"] = self.template_string
        return self.outline_page

    def get_template_name(self):
        """
        Requires subclasses have either a `__template_file__` class property, or override this method
        """
        if self.__template_file__:
            return self.__template_file__
        else:
            raise NotImplementedError
    
    def render(self, **kwargs):
        """
        Renders the widgets to html
        """
        if self.is_template_local:
            return self.render_from_file(**kwargs)
        elif self.template_string:
            return self.render_from_backup_string(**kwargs)
        else:
            raise presalytics.lib.exceptions.MissingConfigException("Missing __template_file__: {}".format(self.get_template_name()))

    def _make_context(self):
        context = {
            "widgets": self.widgets,
            "widget_index": self.WidgetIndexer()
        }
        if self.outline_page.additional_properties:
            context.update(self.outline_page.additional_properties)
            context.pop("template_string", None)
        return context

    def render_from_file(self, **kwargs) -> str:
        """
        Returns rendered html with widgets rendered into template placeholders
        """
        template = self.load_jinja_template()
        context = self._make_context()
        return template.render(**context)

    def render_from_backup_string(self, **kwargs):
        """
        Returns rendered html with widgets rendered into template placeholders
        """
        context = self._make_context()
        options = {
            "loader": jinja2.BaseLoader()
        }
        if context.get("jinja_options", None):
            options.update(context.pop("jinja_options"))
        env = jinja2.Environment(**options)
        env.filters['htmlize'] = htmlize
        template = env.from_string(self.template_string)
        return template.render(**context)

    def read_template_string(self) -> typing.Optional[str]:
        """
        Finds the template file and reads it into a string
        """
        for _dir in self.template_paths:
            fpath = os.path.join(_dir, self.get_template_name())
            if os.path.exists(fpath):
                with open(fpath, 'r') as f:
                    template_string = f.read()
                return template_string
        return None



    def load_jinja_template(self) -> jinja2.Template:
        """
        Uses the fileloader to load a local template file into the jinja2 environment
        """
        loader = jinja2.FileSystemLoader(self.template_paths)
        env = jinja2.Environment(loader=loader)
        env.filters['htmlize'] = htmlize
        return env.get_template(self.get_template_name())

    def check_for_file(self):
        """
        Checks whether `__template_file__` exists locally
        """
        for _dir in self.template_paths:
            fpath = os.path.join(_dir, self.get_template_name())
            if os.path.exists(fpath):
                return True
        return False


class TitleWithSingleItem(JinjaTemplateBuilder):
    """
    Simple single-widget page with a title
    """
    __component_kind__ = 'TitleWithSingleItem'
    __css__ = ['flex_row', 'light_grey', 'responsive_title']
    __template_file__ = 'title_with_single_widget.html'


class TwoUpWithTitle(JinjaTemplateBuilder): 
    """
    View two widgets side-by-side on one page
    """
    __component_kind__ = "TwoUpWithTitle"
    __css__ = ['flex_row', 'light_grey', 'responsive_title']
    __template_file__ = 'two_up_with_title.html'


class BootstrapCustomTemplate(JinjaTemplateBuilder):
    """
    Build customer repsonsive tempaltes using bootstrap to layout widgets
    
    The `presalytics.story.outline.Page` must contain an entry named "template_file" in the its data dictionary.
    The values of the "template_file" varialble must a file path to an html file in the
    current working directory

    Parameters:
    ----------
    name: str
        the page instance name

    page: presalytics.story.outline.Page
        A story outline page object to create the widget from
    
    template_file: str
        A local file path to an html file that wil be used as a template for rendering the page.
        The file path is relative to the current working directory.

    kwargs: dict, optional
        **kwargs can include parameters to pass to the template rendering context. For example, 
        when kwargs is passed `title="An Example Title"`, during rendering, the template's
         `{{title}}` place holder will be replaced with with `An Example Title` 
        
    """
    template_file: str

    __component_kind__ = "bootstrap-custom"

    __plugins__ = [
        {
            'name': 'external_scripts',
            'kind': 'script',
            'config': {
                'approved_scripts_key': 'jquery'
            }
        },
        {
            'name': 'external_scripts',
            'kind': 'script',
            'config': {
                'approved_scripts_key': 'popper'
            }
        },
        {
            'name': 'external_scripts',
            'kind': 'script',
            'config': {
                'approved_scripts_key': 'bootstrap4'
            }
        },
        {
            'name': 'external_links',
            'kind': 'style',
            'config': {
                'approved_styles_key': 'bootstrap4'
            }
        },
    ]


    def __init__(self, page: 'Page', name=None, template_file=None, **kwargs) -> None:
        try:        
            self.template_file = template_file
            if not self.template_file:
                self.template_file = page.additional_properties["template_file"]
        except (KeyError, AttributeError):
            raise presalytics.lib.exceptions.InvalidConfigurationError(message="BootstrapCustomTemplate requires a 'template_file' passed either via 'additional_properties` or a keyword argument")
        super(BootstrapCustomTemplate, self).__init__(page, **kwargs)
        self.outline_page.additional_properties["template_file"] = self.template_file
        self.name = name
        if not self.name:
            self.name = page.name
        self.outline_page.name = self.name
        self.outline_page.kind = self.__component_kind__

    def get_template_name(self):
        return self.template_file

    def serialize(self):
        if self.template_string:
            self.outline_page.additional_properties["template_string"] = self.template_string
        return self.outline_page







import os
import shutil
import pathlib
import unittest
import typing
import presalytics
import io
import lxml
from presalytics.client.api import get_client
if typing.TYPE_CHECKING:
    from presalytics.client.presalytics_story import Story, OoxmlDocument
    from presalytics.client.presalytics_ooxml_automation.models import ChartChartDataDTO, TableTableDataDTO


class TestComponents(unittest.TestCase):
    """
    Test module features thatr render stories to dashboards or other formats
    """

    def test_render_matploytlib_plot(self):
        presalytics.autodiscover_paths.append('test/files')
        presalytics.COMPONENTS = presalytics.story.components.ComponentRegistry(autodiscover_paths=presalytics.autodiscover_paths)
        test_file = os.path.join(os.path.dirname(__file__), 'files', 'matplotlib-outline.yaml')
        outline = presalytics.StoryOutline.import_yaml(test_file)
        html = presalytics.Revealer(outline).package_as_standalone().decode('utf-8')
        self.assertIsInstance(html, str)

    def test_xml_widget(self):
        test_file = os.path.join(os.path.dirname(__file__), "files", "star.pptx")
        tmp_filename = os.path.join(os.path.dirname(__file__), os.path.basename(test_file))
        shutil.copyfile(test_file, tmp_filename)
        client_info = get_client().get_client_info() 
        story = get_client().upload_file_and_await_outline(tmp_filename, include_relationships=True)
        outline = presalytics.StoryOutline.load(story.outline)
        old_widget = outline.pages[0].widgets[0]
        childs = get_client().ooxml_automation.documents_childobjects_get_id(old_widget.data["document_ooxml_id"])
        endpoint_map = presalytics.OoxmlEndpointMap.shape() 
        object_type = endpoint_map.get_object_type()
        info = next(x for x in childs if x.object_type == object_type)
        new_color = {
            "object_name": info.entity_name,
            "hex_color": "FF0000"
        }
        multiparams = {"transforms_list" : [
            {
                'name': 'ChangeShapeColor',
                'function_params': new_color
            },
            {
                'name': 'TextReplace',
                'function_params': {
                    'test_text': "Test Passed!"
                }
            }
        ]}
        widget = presalytics.OoxmlEditorWidget(
            name="test-editor",
            story_id=story.id,
            object_ooxml_id=info.entity_id,
            endpoint_map=endpoint_map,
            transform_class=presalytics.MultiXmlTransform,
            transform_params=multiparams
        )
        outline.pages[0].widgets[0] = widget.outline_widget

        
        presalytics.COMPONENTS.register(widget)
        story.outline = outline.dump()
        get_client().story.story_id_put(story.id, story)
        html = presalytics.Revealer(outline).package_as_standalone().decode('utf-8')
        self.assertIsInstance(html, str)
        os.remove(tmp_filename)

    def test_file_widget(self):
        test_file = os.path.join(os.path.dirname(__file__), "files", "star.pptx")
        tmp_filename = os.path.join(os.path.dirname(__file__), os.path.basename(test_file))
        shutil.copyfile(test_file, tmp_filename)
        story = get_client().upload_file_and_await_outline(tmp_filename, include_relationships=True)
        pathlib.Path(tmp_filename).touch()
        story_outline = presalytics.StoryOutline.load(story.outline)
        widget_data = story_outline.pages[0].widgets[0]
        presalytics.OoxmlFileWidget.deserialize(widget_data)
        html = presalytics.Revealer(story_outline).package_as_standalone().decode('utf-8')
        self.assertIsInstance(html, str)

    def test_bytesio_upload(self):
        test_file = os.path.join(os.path.dirname(__file__), "files", "star.pptx")
        with open(test_file, 'rb') as f:
            _bytes = io.BytesIO(f.read())

        story = presalytics.story_post_file_bytes(get_client(), _bytes, "testfile.pptx")
        from presalytics.client.presalytics_story.models.story import Story
        self.assertIsInstance(story, Story)

    def test_multixml(self):
        test_file = os.path.join(os.path.dirname(__file__), "files", "ooxml_test.xml")
        etree = lxml.etree.parse(test_file)
        multiparams = {"transforms_list" : [
            {
                'name': 'ChangeShapeColor',
                'function_params': {
                    "hex_color": "FF0000"
                }
            },
            {
                'name': 'TextReplace',
                'function_params': {
                    'test_text': "Test Passed!"
                }
            }
        ]}
        inst  = presalytics.MultiXmlTransform(multiparams)
        new_lxml = inst.execute(etree.getroot())
        xml_string = lxml.etree.tostring(new_lxml).decode('utf-8')
        self.assertTrue("Test Passed!" in xml_string)
        self.assertTrue("FF0000" in xml_string)

    def test_chart_update(self):
        story: "Story"

        document: "OoxmlDocument"

        test_file = os.path.join(os.path.dirname(__file__), "files", "bubblechart.pptx")
        story = get_client().upload_file_and_await_outline(test_file, include_relationships=True)
        document = story.ooxml_documents[0]
        object_tree = get_client().ooxml_automation.documents_childobjects_get_id(document.ooxml_automation_id)
        endpoint_map = presalytics.OoxmlEndpointMap.chart() 
        object_type = endpoint_map.get_object_type()
        chart_id = next(entity.entity_id for entity in object_tree if entity.object_type == object_type)
        updater = presalytics.lib.widgets.ooxml.ChartUpdaterWidget('updater', story.id, chart_id)

        dto = updater.get_dto()
        dummySeriesName = "TestSeries1"
        data = [
            [1,2,5,None],
            [None,None,4,5],
            [9,3,8,None],
            [None,None,3,6]
        ]
        
        
        dto.data_points = data
        series = dto.series_names
        series[0] = dummySeriesName
        dto.series_names = series

        df = updater.get_dataframe()
        updater.put_dataframe(df)

        html = updater.to_html()

        self.assertTrue(len(html) > 0)

        widget = updater.serialize()

        self.assertTrue(isinstance(widget, presalytics.story.outline.Widget))

        inst = presalytics.lib.widgets.ooxml.ChartUpdaterWidget.deserialize(widget)

        self.assertTrue(isinstance(inst, presalytics.lib.widgets.ooxml.ChartUpdaterWidget))

    def test_table_update(self):
        story: "Story"
        document: "OoxmlDocument"
        dto: "TableTableDataDTO"

        test_file = os.path.join(os.path.dirname(__file__), "files", "table.pptx")
        story = get_client().upload_file_and_await_outline(test_file, include_relationships=True)
        document = story.ooxml_documents[0]
        object_tree = get_client().ooxml_automation.documents_childobjects_get_id(document.ooxml_automation_id)
        endpoint_map = presalytics.OoxmlEndpointMap.table() 
        object_type = endpoint_map.get_object_type()
        table_id = next(entity.entity_id for entity in object_tree if entity.object_type == object_type)
        updater = presalytics.lib.widgets.ooxml.TableUpdaterWidget('updater', story.id, table_id)

        dto = updater.get_dto()
        dummyEntryName = "CR6!XD?"

        dto.table_data[2][2] = dummyEntryName


        updater.update_from_dto(dto)
        # updater.get_svg_file()
        # client.download_file(story.id, document.ooxml_automation_id)
        self.assertTrue(dummyEntryName in updater.get_svg(updater.table_id))

    def tearDown(self):
        pass



# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class Story(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_at': 'datetime',
        'created_by': 'str',
        'id': 'str',
        'updated_at': 'datetime',
        'updated_by': 'str',
        'collaborators': 'list[StoryCollaborator]',
        'is_public': 'bool',
        'ooxml_documents': 'list[OoxmlDocument]',
        'outline': 'str',
        'outline_history': 'list[StoryOutlineHistory]',
        'revision': 'int',
        'title': 'str'
    }

    attribute_map = {
        'created_at': 'created_at',
        'created_by': 'created_by',
        'id': 'id',
        'updated_at': 'updated_at',
        'updated_by': 'updated_by',
        'collaborators': 'collaborators',
        'is_public': 'is_public',
        'ooxml_documents': 'ooxml_documents',
        'outline': 'outline',
        'outline_history': 'outline_history',
        'revision': 'revision',
        'title': 'title'
    }

    def __init__(self, created_at=None, created_by=None, id=None, updated_at=None, updated_by=None, collaborators=None, is_public=None, ooxml_documents=None, outline=None, outline_history=None, revision=None, title=None):  # noqa: E501
        """Story - a model defined in OpenAPI"""  # noqa: E501

        self._created_at = None
        self._created_by = None
        self._id = None
        self._updated_at = None
        self._updated_by = None
        self._collaborators = None
        self._is_public = None
        self._ooxml_documents = None
        self._outline = None
        self._outline_history = None
        self._revision = None
        self._title = None
        self.discriminator = None

        if created_at is not None:
            self.created_at = created_at
        if created_by is not None:
            self.created_by = created_by
        if id is not None:
            self.id = id
        if updated_at is not None:
            self.updated_at = updated_at
        if updated_by is not None:
            self.updated_by = updated_by
        if collaborators is not None:
            self.collaborators = collaborators
        if is_public is not None:
            self.is_public = is_public
        if ooxml_documents is not None:
            self.ooxml_documents = ooxml_documents
        self.outline = outline
        if outline_history is not None:
            self.outline_history = outline_history
        if revision is not None:
            self.revision = revision
        if title is not None:
            self.title = title

    @property
    def created_at(self):
        """Gets the created_at of this Story.  # noqa: E501


        :return: The created_at of this Story.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Story.


        :param created_at: The created_at of this Story.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def created_by(self):
        """Gets the created_by of this Story.  # noqa: E501


        :return: The created_by of this Story.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this Story.


        :param created_by: The created_by of this Story.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def id(self):
        """Gets the id of this Story.  # noqa: E501


        :return: The id of this Story.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Story.


        :param id: The id of this Story.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def updated_at(self):
        """Gets the updated_at of this Story.  # noqa: E501


        :return: The updated_at of this Story.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Story.


        :param updated_at: The updated_at of this Story.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def updated_by(self):
        """Gets the updated_by of this Story.  # noqa: E501


        :return: The updated_by of this Story.  # noqa: E501
        :rtype: str
        """
        return self._updated_by

    @updated_by.setter
    def updated_by(self, updated_by):
        """Sets the updated_by of this Story.


        :param updated_by: The updated_by of this Story.  # noqa: E501
        :type: str
        """

        self._updated_by = updated_by

    @property
    def collaborators(self):
        """Gets the collaborators of this Story.  # noqa: E501


        :return: The collaborators of this Story.  # noqa: E501
        :rtype: list[StoryCollaborator]
        """
        return self._collaborators

    @collaborators.setter
    def collaborators(self, collaborators):
        """Sets the collaborators of this Story.


        :param collaborators: The collaborators of this Story.  # noqa: E501
        :type: list[StoryCollaborator]
        """

        self._collaborators = collaborators

    @property
    def is_public(self):
        """Gets the is_public of this Story.  # noqa: E501


        :return: The is_public of this Story.  # noqa: E501
        :rtype: bool
        """
        return self._is_public

    @is_public.setter
    def is_public(self, is_public):
        """Sets the is_public of this Story.


        :param is_public: The is_public of this Story.  # noqa: E501
        :type: bool
        """

        self._is_public = is_public

    @property
    def ooxml_documents(self):
        """Gets the ooxml_documents of this Story.  # noqa: E501


        :return: The ooxml_documents of this Story.  # noqa: E501
        :rtype: list[OoxmlDocument]
        """
        return self._ooxml_documents

    @ooxml_documents.setter
    def ooxml_documents(self, ooxml_documents):
        """Sets the ooxml_documents of this Story.


        :param ooxml_documents: The ooxml_documents of this Story.  # noqa: E501
        :type: list[OoxmlDocument]
        """

        self._ooxml_documents = ooxml_documents

    @property
    def outline(self):
        """Gets the outline of this Story.  # noqa: E501


        :return: The outline of this Story.  # noqa: E501
        :rtype: str
        """
        return self._outline

    @outline.setter
    def outline(self, outline):
        """Sets the outline of this Story.


        :param outline: The outline of this Story.  # noqa: E501
        :type: str
        """

        self._outline = outline

    @property
    def outline_history(self):
        """Gets the outline_history of this Story.  # noqa: E501


        :return: The outline_history of this Story.  # noqa: E501
        :rtype: list[StoryOutlineHistory]
        """
        return self._outline_history

    @outline_history.setter
    def outline_history(self, outline_history):
        """Sets the outline_history of this Story.


        :param outline_history: The outline_history of this Story.  # noqa: E501
        :type: list[StoryOutlineHistory]
        """

        self._outline_history = outline_history

    @property
    def revision(self):
        """Gets the revision of this Story.  # noqa: E501


        :return: The revision of this Story.  # noqa: E501
        :rtype: int
        """
        return self._revision

    @revision.setter
    def revision(self, revision):
        """Sets the revision of this Story.


        :param revision: The revision of this Story.  # noqa: E501
        :type: int
        """

        self._revision = revision

    @property
    def title(self):
        """Gets the title of this Story.  # noqa: E501


        :return: The title of this Story.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this Story.


        :param title: The title of this Story.  # noqa: E501
        :type: str
        """

        self._title = title

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Story):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
